{% load djblets_utils %}

{% if user.is_authenticated %}
{%  definevar "icon_class" %}rb-icon-star-{% if starred %}on{% else %}off{% endif %}{% enddefinevar %}
<div class="rb-icon star {{icon_class}}" data-starred="{{starred}}" data-object-type="{{object.type}}" data-object-id="{{object.id}}"></div>
{% endif %}
