{% extends "datagrid/listview.html" %}
{% load i18n reviewtags %}

{% block datagrid_title %}
{% if datagrid.group %}{% star datagrid.group %}{% endif %}
{{block.super}}
 <ul>
{% if datagrid.show_closed %}
  <li><a href="?show-closed=0">{% trans "Hide closed" %}</a></li>
{% else %}
  <li><a href="?show-closed=1">{% trans "Show closed" %}</a></li>
{% endif %}
 </ul>
{% endblock %}
