{% extends "base.html" %}
{% load djblets_deco i18n %}

{% block title %}{% trans "Recover Password" %}{% endblock %}

{% block content %}
{%  box "passwordreset" %}
 <form method="post" action=".">
  {% csrf_token %}

  <div class="box-head">
   <h1 class="box-title">{% trans "Recover Password" %}</h1>
  </div>
  <div class="box-main">
   <p>{% blocktrans %}Forgotten your password? Enter your e-mail address, and
   we'll send you instructions for creating a new one.{% endblocktrans %}</p>
   <table>
    <tr>
     <td class="label"><label for="id_email">{% trans "E-mail address" %}</label></td>
     <td class="field"><input type="text" name="email" id="id_email" /></td>
     <td class="field"><input type="submit" value="{% trans "Recover" %}" /></td>
    </tr>
   </table>
  </div>
 </form>
{%  endbox %}
{% endblock %}
