{% load i18n rb_extensions %}

<ul id="navbar">
{% if request.user.is_authenticated or not siteconfig_settings.auth_require_sitewide_login %}
{%  if request.user.is_authenticated %}
 <li><a href="{% url 'dashboard' %}">{% trans "My Dashboard" %}</a></li>
 <li><a href="{% url 'new-review-request' %}">{% trans "New Review Request" %}</a></li>
{%  endif %}
 <li><a href="{% url 'all-review-requests' %}">{% trans "All Review Requests" %}</a></li>
 <li><a href="{% url 'all-users' %}">{% trans "Users" %}</a></li>
 <li><a href="{% url 'all-groups' %}">{% trans "Groups" %}</a></li>
{%  navigation_bar_hooks %}
{% endif %}
</ul>
