<li class="item has-url{% if active %} active{% endif %}{% if has_count %} has-count{% endif %}{% if css_classes %} {{css_classes|join:' '}}{% endif %}" onclick="javascript:window.location=&quot;{{url}}&quot;; return false;">
 <div class="datagrid-sidebar-row">
  <div class="rb-icon {{icon_name|default:''}}"></div>
  <span class="label"><a href="{{url}}">{{label}}</a></span>
{% if has_count %}
  <span class="count {% if count == 0 %}count-zero{% endif %}">{{count}}</span>
{% endif %}
 </div>
</li>
