{% extends "accounts/login.html" %}
{% load compressed i18n %}

{% block title %}{{title}} | {% trans "Administration" %}{% endblock %}

{% block css %}
{%  compressed_css "admin" %}
{% endblock %}

{% block bodytag %}
<body class="admin login">
{% endblock %}

 {% block hidden_fields %}
 {% csrf_token %}
 <input type="hidden" name="this_is_the_login_form" value="1" />
 <input type="hidden" name="next" value="{{next}}" />
 {% endblock %}
