/* RB Colors */
/*
 * At least on Chrome, using points instead of pixels does not reliably
 * result in identical sizings when there are elements positioned inside
 * a <pre>. Pixels fixes this.
 */
/* Diff styles */
/* !(*%!(&^ (see http://www.positioniseverything.net/easyclearing.html) */
.clearfix {
  display: inline-block;
}
.clearfix:after {
  clear: both;
  content: ".";
  display: block;
  height: 0;
  visibility: hidden;
}
/* Hides from IE-mac \*/
* html .clearfix {
  height: 1%;
}
.clearfix {
  display: block;
}
/* End hide from IE-mac */
#new-review-request {
  border: 1px solid #888a85;
  width: 100%;
  height: 55em;
  display: none;
}
#new-review-request .repository-selector {
  float: left;
  width: 20em;
  height: 55em;
  border-right: 1px #d0d0d0 solid;
  background-color: #333333;
  color: white;
  overflow-y: auto;
  overflow-x: hidden;
  position: relative;
}
#new-review-request .repository-selector h3 {
  padding: 0;
  margin: 1em;
  overflow: hidden;
}
#new-review-request .repository-selector .repository {
  cursor: pointer;
  width: 18em;
  padding: 1em;
}
#new-review-request .repository-selector .repository.selected {
  background-color: #444444;
  background-repeat: no-repeat;
  background-image: -webkit-gradient(linear, 0 0, 0 100%, from(#444444), color-stop(15%, #555555), to(#444444));
  background-image: -webkit-linear-gradient(#444444, #555555 15%, #444444);
  background-image: -moz-linear-gradient(top, #444444, #555555 15%, #444444);
  background-image: -ms-linear-gradient(#444444, #555555 15%, #444444);
  background-image: -o-linear-gradient(#444444, #555555 15%, #444444);
  background-image: linear-gradient(#444444, #555555 15%, #444444);
  filter: progid:DXImageTransform.Microsoft.gradient(startColorstr='#444444', endColorstr='#444444', GradientType=0);
}
#new-review-request .repository-selector .search-icon-wrapper {
  position: absolute;
  top: 0;
  background-color: #333333;
  z-index: 90;
}
#new-review-request .repository-selector .repository-search-icon {
  margin: 9px;
}
#new-review-request .repository-selector .repository-search {
  position: absolute;
  right: 8px;
  top: 0.75em;
  width: 0;
  visibility: hidden;
  background-color: #4d4d4d;
  border: 1px solid #999;
  color: #ccc;
  -moz-box-shadow: none;
  -webkit-box-shadow: none;
  box-shadow: none;
  padding: 2px;
  z-index: 91;
}
#new-review-request .main {
  height: 55em;
  overflow: scroll;
  margin-left: 20em;
  background-color: #f4f4f4;
}
#new-review-request .main .hint {
  background-color: white;
  height: 55em;
  line-height: 55em;
  width: 100%;
  vertical-align: middle;
  text-align: center;
}
#new-review-request .pre-commit {
  background-color: #f4f4f4;
}
#new-review-request .pre-commit .tip {
  margin: 2em 2em 1em 2em;
  text-align: center;
}
#new-review-request .pre-commit #prompt-for-basedir,
#new-review-request .pre-commit #prompt-for-diff,
#new-review-request .pre-commit #prompt-for-parent-diff,
#new-review-request .pre-commit #prompt-for-change-number,
#new-review-request .pre-commit #processing-diff,
#new-review-request .pre-commit #uploading-diffs,
#new-review-request .pre-commit #error-indicator {
  display: none;
}
#new-review-request .pre-commit #prompt-for-parent-diff {
  line-height: 6em;
}
#new-review-request .pre-commit #prompt-for-parent-diff form {
  margin-top: 4em;
}
#new-review-request .pre-commit #error-contents,
#new-review-request .pre-commit #parent-diff-error-contents {
  position: relative;
  margin: 0 10em;
  line-height: 12pt;
  margin-bottom: 1em;
}
#new-review-request .pre-commit #error-contents img,
#new-review-request .pre-commit #parent-diff-error-contents img {
  margin-right: 5px;
  vertical-align: sub;
}
#new-review-request .pre-commit #processing-diff,
#new-review-request .pre-commit #uploading-diffs {
  background: url(../images/spinner.gif) center center no-repeat;
}
#new-review-request .pre-commit .input {
  text-align: center;
  vertical-align: middle;
  height: 14em;
  line-height: 14em;
  margin: 2em 2em 6em 2em;
  border: 1px solid #d0d0d0;
  -moz-border-radius: 6px;
  -webkit-border-radius: 6px;
  border-radius: 6px;
  background-color: #ffffff;
  position: relative;
}
#new-review-request .pre-commit .input .startover {
  position: absolute;
  bottom: 0;
  right: 5px;
  line-height: 14pt;
}
#new-review-request .pre-commit .drag-hover {
  border: 1px solid #c6c6c6;
  -moz-box-shadow: inset 1px 1px 2px rgba(0, 0, 0, 0.15);
  -webkit-box-shadow: inset 1px 1px 2px rgba(0, 0, 0, 0.15);
  box-shadow: inset 1px 1px 2px rgba(0, 0, 0, 0.15);
}
#new-review-request .section-header {
  padding: 0.5em;
  border-bottom: 1px #d0d0d0 solid;
  font-size: 12pt;
  text-align: center;
  background-color: #c6dcf3;
}
#new-review-request .branches select {
  vertical-align: top;
}
#new-review-request .commit {
  border-bottom: 1px #d0d0d0 solid;
  position: relative;
  padding: 1em;
  cursor: pointer;
}
#new-review-request .commit .summary {
  font-size: 10pt;
  font-weight: 600;
  width: 52em;
  padding-bottom: 2px;
  overflow: hidden;
  white-space: nowrap;
  text-overflow: ellipsis;
  vertical-align: bottom;
}
#new-review-request .commit .jump-to-commit {
  position: absolute;
  right: 20px;
  top: 50%;
  margin-top: -9px;
}
#new-review-request .commit .progress {
  display: none;
  position: absolute;
  width: 16px;
  right: 1em;
  top: 0;
  height: 100%;
  background: url(../images/spinner.gif) center center no-repeat;
}
#new-review-request .commit:nth-child(even) {
  background-color: #efefef;
}
#new-review-request .commit:nth-child(odd) {
  background-color: #e5e5e5;
}
#new-review-request .commit:hover {
  background-color: #f9f9f9;
}
#new-review-request .post-commit {
  border-bottom: 1px #d0d0d0 solid;
}
#new-review-request .post-commit .error {
  background: #FFE9E9;
  padding: 1em;
  font-size: 110%;
}
#new-review-request .post-commit .error a {
  color: blue;
}
#new-review-request .post-commit .error .error-text {
  color: #CC0000;
}
#new-review-request .files-only {
  margin: 14em;
  text-align: center;
}
#new-review-request .files-only input {
  margin-top: 2em;
}
