/* RB Colors */
/*
 * At least on Chrome, using points instead of pixels does not reliably
 * result in identical sizings when there are elements positioned inside
 * a <pre>. Pixels fixes this.
 */
/* Diff styles */
/* !(*%!(&^ (see http://www.positioniseverything.net/easyclearing.html) */
.clearfix {
  display: inline-block;
}
.clearfix:after {
  clear: both;
  content: ".";
  display: block;
  height: 0;
  visibility: hidden;
}
/* Hides from IE-mac \*/
* html .clearfix {
  height: 1%;
}
.clearfix {
  display: block;
}
/* End hide from IE-mac */
/* TODO: Reuse the colors from diffviewer.less, once they're moved out. */
.image-review-ui-diff-thumbnail .orig-image {
  border: 1px #ff7777 solid;
  display: inline-block;
}
.image-review-ui-diff-thumbnail .modified-image {
  border: 1px #63c363 solid;
  display: inline-block;
  margin-left: 5px;
}
.image-review-ui {
  background: white;
  padding: 2em;
  position: relative;
  -moz-border-radius: 0 0 6px 6px;
  -webkit-border-radius: 0 0 6px 6px;
  border-radius: 0 0 6px 6px;
}
.review-ui-inline .image-review-ui {
  padding: 0;
}
.image-review-ui img {
  display: block;
  margin: 0 auto;
  padding: 0;
}
.image-review-ui .caption {
  margin-bottom: 1em;
  text-align: center;
}
.image-review-ui .image-diffs {
  padding: 2em;
  text-align: center;
}
.image-review-ui .image-diffs .image-slider {
  margin: 2em auto 0 auto;
  width: 200px;
}
.image-review-ui .image-diffs .orig-image {
  border: 1px #ff7777 solid;
}
.image-review-ui .image-diffs .modified-image {
  border: 1px #63c363 solid;
}
.image-review-ui .image-diffs .image-container {
  display: inline-block;
  float: none;
  margin: 0 2em;
}
.image-review-ui .image-diffs .image-diff-difference canvas {
  border: 1px #CCCCCC solid;
}
.image-review-ui .image-diffs .image-diff-two-up img {
  display: inline-block;
}
.image-review-ui .image-diffs .image-diff-two-up .image-container-orig {
  right: 50%;
}
.image-review-ui .image-diffs .image-diff-two-up .image-container-modified {
  left: 50%;
}
.image-review-ui .image-diffs .image-diff-onion .image-containers,
.image-review-ui .image-diffs .image-diff-split .image-containers {
  margin: auto;
  position: relative;
}
.image-review-ui .image-diffs .image-diff-onion .image-containers .orig-image,
.image-review-ui .image-diffs .image-diff-split .image-containers .orig-image,
.image-review-ui .image-diffs .image-diff-onion .image-containers .modified-image,
.image-review-ui .image-diffs .image-diff-split .image-containers .modified-image {
  display: block;
  margin: 0;
  position: absolute;
}
.image-review-ui .image-diff-split-container-orig,
.image-review-ui .image-diff-split-container-modified {
  overflow: hidden;
  position: absolute;
}
.image-review-ui .image-diff-split-container-orig .orig-image {
  left: 0;
}
.image-review-ui .image-diff-split-container-modified {
  border-left: 1px solid #999999;
  right: 0;
}
.image-review-ui .image-diff-split-container-modified .modified-image {
  right: 0;
}
.image-review-ui .image-diff-modes {
  background: #e8e8e8;
  border-bottom: 1px #c2c2c2 solid;
  list-style: none;
  margin: 0;
  padding: 1em;
  text-align: center;
}
.image-review-ui .image-diff-modes li {
  display: inline-block;
  margin: 0 1em;
  padding: 0;
  text-align: center;
}
.image-review-ui .image-diff-modes li.selected a {
  font-weight: bold;
  color: #333;
}
.image-review-ui .image-diff-modes li a {
  color: #444;
  padding: 1em 0;
}
.image-review-ui .selection {
  border: 1px #a1c0e3 solid;
  position: absolute;
}
.image-review-ui .selection.draft {
  border: 1px #aceb6f solid;
}
.image-review-ui .selection.draft .selection-flag {
  background: #aceb6f;
}
.image-review-ui .selection-flag {
  background-color: #a1c0e3;
  cursor: pointer;
  height: 2em;
  left: 0px;
  line-height: 170%;
  text-align: center;
  top: 0px;
  width: 2em;
}
.image-review-ui .selection-container {
  cursor: crosshair;
  position: absolute;
  margin: 0;
  z-index: 30;
}
.image-review-ui .selection-rect {
  border: 1px #ff0000 dashed;
  position: absolute;
}
