{% load djblets_utils %}
{% definevar "precss" %}white-space: pre-wrap; white-space: -moz-pre-wrap; white-space: -pre-wrap; white-space: -o-pre-wrap; word-wrap: break-word;{% enddefinevar %}
{% if not embedded %}
<html>
 <body>
  <div style="font-family: Verdana, Arial, Helvetica, Sans-Serif;">
   <table bgcolor="#f9f3c9" width="100%" cellpadding="12" style="border: 1px #c9c399 solid; border-radius: 6px; -moz-border-radius: 6px; -webkit-border-radius: 6px;">
    <tr>
     <td>
      This is an automatically generated e-mail.
     </td>
    </tr>
   </table>
   <br />
{% endif %}

  <div>
   <table bgcolor="#f0f0f0" cellpadding="5" cellspacing="5" style="border: 1px solid #c0c0c0; margin-bottom: 10px; border-radius: 6px; -moz-border-radius: 6px; -webkit-border-radius: 6px;">
    <tr>
     <td>
{{user.username}} has registered on <a href="{{domain_method}}://{{domain}}">{{domain_method}}://{{domain}}</a> on {{user.date_joined|date:"F jS, Y, P T"}}.
<br/>
If you want to grant certain permissions for this user, please visit <a href="{{domain_method}}://{{domain}}{{user_url}}">{{domain_method}}://{{domain}}{{user_url}}</a>.
     </td>
    </tr>
   </table>
  </div>
{% if not embedded %}
  </div>
 </body>
</html>
{% endif %}
