{% load i18n %}
{% if widget.data.groups %}
<table style="width: 100%;">
{% for group in widget.data.groups %}
 <tr>
  <th><a href="db/reviews/group/{{group.id}}">{{group.display_name}}</a></th>
  <td>{{group.users.count}}</td>
 </tr>
{% endfor %}
</table>
{% else %}
<p class="no-result">{% trans "No Groups Available." %}</p>
<p><a href="db/reviews/group/add/">Add +</a></p>
{% endif %}
