{% extends "base.html" %}
{% load compressed i18n %}

{% block ie6_csshacks %}
   #review-banner .banner { position: absolute; }
{% endblock %}

{% block css %}
{% compressed_css "reviews" %}
{% endblock %}

{% block scripts-post %}
{%  compressed_js "reviews" %}
{% endblock %}

{% block bodytag %}
{{block.super}}

{%  block review_banner %}
<div id="review-banner"{% if not review %} style="display: none;"{% endif %}>
 <div class="banner">
  <h1>{% trans "You have a pending review." %}</h1>
  <input id="review-banner-edit" type="button" value="{% trans "Edit Review" %}" />
  <input id="review-banner-publish" type="button" value="{% trans "Publish" %}" />
  <input id="review-banner-discard" type="button" value="{% trans "Discard" %}" />
 </div>
</div>
{%  endblock %}
{% endblock %}
