{% extends "base.html" %}
{% load djblets_deco i18n %}

{% block title %}{% trans "Password reset complete" %}{% endblock %}

{% block content %}
{%  box "passwordreset" %}
<div class="box-head">
 <h1 class="box-title">{% trans "Password reset complete" %}</h1>
</div>
<div class="box-main">
 <p>{% trans "Your password has been set.  You may go ahead and log in now." %}</p>
 <p><a href="{% url 'login' %}">{% trans "Log in." %}</a></p>
</div>
{%  endbox %}
{% endblock %}
