{% extends "admin/base_site.html" %}
{% load compressed i18n log rbadmintags staticfiles %}

{% block coltype %}colM{% endblock %}
{% block bodyclass %}dashboard{% endblock %}
{% block breadcrumbs %}{% endblock %}

{% block scripts-post %}
<!--[if IE]><script src="{% static "lib/js/flot/excanvas.min.js" %}"></script><![endif]-->
{% compressed_js "admin" %}
{% endblock %}

{% block content_title %}{% endblock %}

{% block content %}
<div id="admin-dashboard">
 <div id="dashboard-view">
  <div id="admin-widgets">
{% for widget in primary_widgets %}
   {% admin_widget widget %}
{% endfor %}
  </div>
  <div id="admin-extras">
{% for widget in secondary_widgets %}
   {% admin_widget widget %}
{% endfor %}
  </div>
 </div>
</div>

<br class="clear" />
{% endblock %}
