#search_results {
  margin: 3em;
}
#search_results .paginator {
  border-top: 1px #E0E0E0 solid;
  font-size: 120%;
  margin-top: 2em;
  padding-top: 2em;
}
#search_results .paginator a {
  color: blue;
  text-decoration: none;
}
#search_results .paginator a,
#search_results .paginator b {
  margin-right: 0.5em;
}
#search_results .search-results-filters {
  border-bottom: 1px #cccccc solid;
  margin-bottom: 2em;
}
#search_results .search-results-filters a {
  color: #666;
  display: inline-block;
  font-size: 130%;
  text-decoration: none;
  padding: 0.5em 1em;
  border: 1px transparent solid;
  border-top-width: 2px;
}
#search_results .search-results-filters a.active {
  color: black;
  border: 1px #cccccc solid;
  border-bottom: 1px white solid;
  border-top-width: 2px;
}
#search_results .search-results-stats {
  color: #444444;
  font-size: 110%;
  padding-bottom: 2em;
  margin-left: 1em;
}
#search_results .search-result {
  display: inline-block;
  display: block;
  line-height: 1.5;
  margin-bottom: 2em;
}
#search_results .search-result:after {
  clear: both;
  content: ".";
  display: block;
  height: 0;
  visibility: hidden;
}
#search_results .search-result a {
  text-decoration: none;
}
#search_results .search-result h2 {
  font-size: 140%;
  font-weight: normal;
  margin: 0;
}
#search_results .search-result .search-result-by {
  color: #555555;
  font-size: 85%;
  margin-top: 0.3em;
}
#search_results .search-result .search-result-body {
  float: left;
}
#search_results .search-result .search-result-excerpt {
  margin-top: 0.3em;
  width: 60em;
}
#search_results .search-result .search-result-icon {
  float: left;
  margin-right: 1em;
}
#search_results .search-result .search-result-links {
  margin-top: 1em;
}
#search_results .search-result .search-result-username {
  font-size: 120%;
}
#search_results .search-result .search-result-user-groups {
  margin-top: 1em;
}
