/* BASICS */

.CodeMirror {
  /* Set height, width, borders, and global font properties here */
  font-family: monospace;
  height: 300px;
}
.CodeMirror-scroll {
  /* Set scrolling behaviour here */
  overflow: auto;
}

/* PADDING */

.CodeMirror-lines {
  padding: 4px 0; /* Vertical padding around content */
}
.CodeMirror pre {
  padding: 0 4px; /* Horizontal padding of content */
}

.CodeMirror-scrollbar-filler, .CodeMirror-gutter-filler {
  background-color: white; /* The little square between H and V scrollbars */
}

/* GUTTER */

.CodeMirror-gutters {
  border-right: 1px solid #ddd;
  background-color: #f7f7f7;
  white-space: nowrap;
}
.CodeMirror-linenumbers {}
.CodeMirror-linenumber {
  padding: 0 3px 0 5px;
  min-width: 20px;
  text-align: right;
  color: #999;
}

/* CURSOR */

.CodeMirror div.CodeMirror-cursor {
  border-left: 1px solid black;
  z-index: 3;
}
/* Shown when moving in bi-directional text */
.CodeMirror div.CodeMirror-secondarycursor {
  border-left: 1px solid silver;
}
.CodeMirror.cm-keymap-fat-cursor div.CodeMirror-cursor {
  width: auto;
  border: 0;
  background: #7e7;
  z-index: 1;
}
/* Can style cursor different in overwrite (non-insert) mode */
.CodeMirror div.CodeMirror-cursor.CodeMirror-overwrite {}

.cm-tab { display: inline-block; }

/* DEFAULT THEME */

.cm-s-default .cm-keyword {color: #708;}
.cm-s-default .cm-atom {color: #219;}
.cm-s-default .cm-number {color: #164;}
.cm-s-default .cm-def {color: #00f;}
.cm-s-default .cm-variable {color: black;}
.cm-s-default .cm-variable-2 {color: #05a;}
.cm-s-default .cm-variable-3 {color: #085;}
.cm-s-default .cm-property {color: black;}
.cm-s-default .cm-operator {color: black;}
.cm-s-default .cm-comment {color: #a50;}
.cm-s-default .cm-string {color: #a11;}
.cm-s-default .cm-string-2 {color: #f50;}
.cm-s-default .cm-meta {color: #555;}
.cm-s-default .cm-qualifier {color: #555;}
.cm-s-default .cm-builtin {color: #30a;}
.cm-s-default .cm-bracket {color: #997;}
.cm-s-default .cm-tag {color: #170;}
.cm-s-default .cm-attribute {color: #00c;}
.cm-s-default .cm-header {color: blue;}
.cm-s-default .cm-quote {color: #090;}
.cm-s-default .cm-hr {color: #999;}
.cm-s-default .cm-link {color: #00c;}

.cm-negative {color: #d44;}
.cm-positive {color: #292;}
.cm-header, .cm-strong {font-weight: bold;}
.cm-em {font-style: italic;}
.cm-link {text-decoration: underline;}

.cm-s-default .cm-error {color: #f00;}
.cm-invalidchar {color: #f00;}

div.CodeMirror span.CodeMirror-matchingbracket {color: #0f0;}
div.CodeMirror span.CodeMirror-nonmatchingbracket {color: #f22;}
.CodeMirror-activeline-background {background: #e8f2ff;}

/* STOP */

/* The rest of this file contains styles related to the mechanics of
   the editor. You probably shouldn't touch them. */

.CodeMirror {
  line-height: 1;
  position: relative;
  overflow: hidden;
  background: white;
  color: black;
}

.CodeMirror-scroll {
  /* 30px is the magic margin used to hide the element's real scrollbars */
  /* See overflow: hidden in .CodeMirror */
  margin-bottom: -30px; margin-right: -30px;
  padding-bottom: 30px; padding-right: 30px;
  height: 100%;
  outline: none; /* Prevent dragging from highlighting the element */
  position: relative;
  -moz-box-sizing: content-box;
  box-sizing: content-box;
}
.CodeMirror-sizer {
  position: relative;
}

/* The fake, visible scrollbars. Used to force redraw during scrolling
   before actuall scrolling happens, thus preventing shaking and
   flickering artifacts. */
.CodeMirror-vscrollbar, .CodeMirror-hscrollbar, .CodeMirror-scrollbar-filler, .CodeMirror-gutter-filler {
  position: absolute;
  z-index: 6;
  display: none;
}
.CodeMirror-vscrollbar {
  right: 0; top: 0;
  overflow-x: hidden;
  overflow-y: scroll;
}
.CodeMirror-hscrollbar {
  bottom: 0; left: 0;
  overflow-y: hidden;
  overflow-x: scroll;
}
.CodeMirror-scrollbar-filler {
  right: 0; bottom: 0;
}
.CodeMirror-gutter-filler {
  left: 0; bottom: 0;
}

.CodeMirror-gutters {
  position: absolute; left: 0; top: 0;
  padding-bottom: 30px;
  z-index: 3;
}
.CodeMirror-gutter {
  white-space: normal;
  height: 100%;
  -moz-box-sizing: content-box;
  box-sizing: content-box;
  padding-bottom: 30px;
  margin-bottom: -32px;
  display: inline-block;
  /* Hack to make IE7 behave */
  *zoom:1;
  *display:inline;
}
.CodeMirror-gutter-elt {
  position: absolute;
  cursor: default;
  z-index: 4;
}

.CodeMirror-lines {
  cursor: text;
}
.CodeMirror pre {
  /* Reset some styles that the rest of the page might have set */
  -moz-border-radius: 0; -webkit-border-radius: 0; border-radius: 0;
  border-width: 0;
  background: transparent;
  font-family: inherit;
  font-size: inherit;
  margin: 0;
  white-space: pre;
  word-wrap: normal;
  line-height: inherit;
  color: inherit;
  z-index: 2;
  position: relative;
  overflow: visible;
}
.CodeMirror-wrap pre {
  word-wrap: break-word;
  white-space: pre-wrap;
  word-break: normal;
}
.CodeMirror-code pre {
  border-right: 30px solid transparent;
  width: -webkit-fit-content;
  width: -moz-fit-content;
  width: fit-content;
}
.CodeMirror-wrap .CodeMirror-code pre {
  border-right: none;
  width: auto;
}
.CodeMirror-linebackground {
  position: absolute;
  left: 0; right: 0; top: 0; bottom: 0;
  z-index: 0;
}

.CodeMirror-linewidget {
  position: relative;
  z-index: 2;
  overflow: auto;
}

.CodeMirror-widget {}

.CodeMirror-wrap .CodeMirror-scroll {
  overflow-x: hidden;
}

.CodeMirror-measure {
  position: absolute;
  width: 100%;
  height: 0;
  overflow: hidden;
  visibility: hidden;
}
.CodeMirror-measure pre { position: static; }

.CodeMirror div.CodeMirror-cursor {
  position: absolute;
  visibility: hidden;
  border-right: none;
  width: 0;
}
.CodeMirror-focused div.CodeMirror-cursor {
  visibility: visible;
}

.CodeMirror-selected { background: #d9d9d9; }
.CodeMirror-focused .CodeMirror-selected { background: #d7d4f0; }

.cm-searching {
  background: #ffa;
  background: rgba(255, 255, 0, .4);
}

/* IE7 hack to prevent it from returning funny offsetTops on the spans */
.CodeMirror span { *vertical-align: text-bottom; }

@media print {
  /* Hide the cursor when printing */
  .CodeMirror div.CodeMirror-cursor {
    visibility: hidden;
  }
}

/*! jQuery UI - v1.8.24 - 2012-09-28
* https://github.com/jquery/jquery-ui
* Includes: jquery.ui.core.css, jquery.ui.accordion.css, jquery.ui.autocomplete.css, jquery.ui.button.css, jquery.ui.datepicker.css, jquery.ui.dialog.css, jquery.ui.progressbar.css, jquery.ui.resizable.css, jquery.ui.selectable.css, jquery.ui.slider.css, jquery.ui.tabs.css, jquery.ui.theme.css
* Copyright (c) 2012 AUTHORS.txt; Licensed MIT, GPL */
.ui-helper-hidden{display:none}.ui-helper-hidden-accessible{position:absolute!important;clip:rect(1px);clip:rect(1px,1px,1px,1px)}.ui-helper-reset{margin:0;padding:0;border:0;outline:0;line-height:1.3;text-decoration:none;font-size:100%;list-style:none}.ui-helper-clearfix:before,.ui-helper-clearfix:after{content:"";display:table}.ui-helper-clearfix:after{clear:both}.ui-helper-clearfix{zoom:1}.ui-helper-zfix{width:100%;height:100%;top:0;left:0;position:absolute;opacity:0;filter:Alpha(Opacity=0)}.ui-state-disabled{cursor:default!important}.ui-icon{display:block;text-indent:-99999px;overflow:hidden;background-repeat:no-repeat}.ui-widget-overlay{position:absolute;top:0;left:0;width:100%;height:100%}.ui-accordion{width:100%}.ui-accordion .ui-accordion-header{cursor:pointer;position:relative;margin-top:1px;zoom:1}.ui-accordion .ui-accordion-li-fix{display:inline}.ui-accordion .ui-accordion-header-active{border-bottom:0!important}.ui-accordion .ui-accordion-header a{display:block;font-size:1em;padding:.5em .5em .5em .7em}.ui-accordion-icons .ui-accordion-header a{padding-left:2.2em}.ui-accordion .ui-accordion-header .ui-icon{position:absolute;left:.5em;top:50%;margin-top:-8px}.ui-accordion .ui-accordion-content{padding:1em 2.2em;border-top:0;margin-top:-2px;position:relative;top:1px;margin-bottom:2px;overflow:auto;display:none;zoom:1}.ui-accordion .ui-accordion-content-active{display:block}.ui-autocomplete{position:absolute;cursor:default}* html .ui-autocomplete{width:1px}.ui-menu{list-style:none;padding:2px;margin:0;display:block;float:left}.ui-menu .ui-menu{margin-top:-3px}.ui-menu .ui-menu-item{margin:0;padding:0;zoom:1;float:left;clear:left;width:100%}.ui-menu .ui-menu-item a{text-decoration:none;display:block;padding:.2em .4em;line-height:1.5;zoom:1}.ui-menu .ui-menu-item a.ui-state-hover,.ui-menu .ui-menu-item a.ui-state-active{font-weight:normal;margin:-1px}.ui-button{display:inline-block;position:relative;padding:0;margin-right:.1em;text-decoration:none!important;cursor:pointer;text-align:center;zoom:1;overflow:visible}.ui-button-icon-only{width:2.2em}button.ui-button-icon-only{width:2.4em}.ui-button-icons-only{width:3.4em}button.ui-button-icons-only{width:3.7em}.ui-button .ui-button-text{display:block;line-height:1.4}.ui-button-text-only .ui-button-text{padding:.4em 1em}.ui-button-icon-only .ui-button-text,.ui-button-icons-only .ui-button-text{padding:.4em;text-indent:-9999999px}.ui-button-text-icon-primary .ui-button-text,.ui-button-text-icons .ui-button-text{padding:.4em 1em .4em 2.1em}.ui-button-text-icon-secondary .ui-button-text,.ui-button-text-icons .ui-button-text{padding:.4em 2.1em .4em 1em}.ui-button-text-icons .ui-button-text{padding-left:2.1em;padding-right:2.1em}input.ui-button{padding:.4em 1em}.ui-button-icon-only .ui-icon,.ui-button-text-icon-primary .ui-icon,.ui-button-text-icon-secondary .ui-icon,.ui-button-text-icons .ui-icon,.ui-button-icons-only .ui-icon{position:absolute;top:50%;margin-top:-8px}.ui-button-icon-only .ui-icon{left:50%;margin-left:-8px}.ui-button-text-icon-primary .ui-button-icon-primary,.ui-button-text-icons .ui-button-icon-primary,.ui-button-icons-only .ui-button-icon-primary{left:.5em}.ui-button-text-icon-secondary .ui-button-icon-secondary,.ui-button-text-icons .ui-button-icon-secondary,.ui-button-icons-only .ui-button-icon-secondary{right:.5em}.ui-button-text-icons .ui-button-icon-secondary,.ui-button-icons-only .ui-button-icon-secondary{right:.5em}.ui-buttonset{margin-right:7px}.ui-buttonset .ui-button{margin-left:0;margin-right:-.3em}button.ui-button::-moz-focus-inner{border:0;padding:0}.ui-datepicker{width:17em;padding:.2em .2em 0;display:none}.ui-datepicker .ui-datepicker-header{position:relative;padding:.2em 0}.ui-datepicker .ui-datepicker-prev,.ui-datepicker .ui-datepicker-next{position:absolute;top:2px;width:1.8em;height:1.8em}.ui-datepicker .ui-datepicker-prev-hover,.ui-datepicker .ui-datepicker-next-hover{top:1px}.ui-datepicker .ui-datepicker-prev{left:2px}.ui-datepicker .ui-datepicker-next{right:2px}.ui-datepicker .ui-datepicker-prev-hover{left:1px}.ui-datepicker .ui-datepicker-next-hover{right:1px}.ui-datepicker .ui-datepicker-prev span,.ui-datepicker .ui-datepicker-next span{display:block;position:absolute;left:50%;margin-left:-8px;top:50%;margin-top:-8px}.ui-datepicker .ui-datepicker-title{margin:0 2.3em;line-height:1.8em;text-align:center}.ui-datepicker .ui-datepicker-title select{font-size:1em;margin:1px 0}.ui-datepicker select.ui-datepicker-month-year{width:100%}.ui-datepicker select.ui-datepicker-month,.ui-datepicker select.ui-datepicker-year{width:49%}.ui-datepicker table{width:100%;font-size:.9em;border-collapse:collapse;margin:0 0 .4em}.ui-datepicker th{padding:.7em .3em;text-align:center;font-weight:bold;border:0}.ui-datepicker td{border:0;padding:1px}.ui-datepicker td span,.ui-datepicker td a{display:block;padding:.2em;text-align:right;text-decoration:none}.ui-datepicker .ui-datepicker-buttonpane{background-image:none;margin:.7em 0 0 0;padding:0 .2em;border-left:0;border-right:0;border-bottom:0}.ui-datepicker .ui-datepicker-buttonpane button{float:right;margin:.5em .2em .4em;cursor:pointer;padding:.2em .6em .3em .6em;width:auto;overflow:visible}.ui-datepicker .ui-datepicker-buttonpane button.ui-datepicker-current{float:left}.ui-datepicker.ui-datepicker-multi{width:auto}.ui-datepicker-multi .ui-datepicker-group{float:left}.ui-datepicker-multi .ui-datepicker-group table{width:95%;margin:0 auto .4em}.ui-datepicker-multi-2 .ui-datepicker-group{width:50%}.ui-datepicker-multi-3 .ui-datepicker-group{width:33.3%}.ui-datepicker-multi-4 .ui-datepicker-group{width:25%}.ui-datepicker-multi .ui-datepicker-group-last .ui-datepicker-header{border-left-width:0}.ui-datepicker-multi .ui-datepicker-group-middle .ui-datepicker-header{border-left-width:0}.ui-datepicker-multi .ui-datepicker-buttonpane{clear:left}.ui-datepicker-row-break{clear:both;width:100%;font-size:0em}.ui-datepicker-rtl{direction:rtl}.ui-datepicker-rtl .ui-datepicker-prev{right:2px;left:auto}.ui-datepicker-rtl .ui-datepicker-next{left:2px;right:auto}.ui-datepicker-rtl .ui-datepicker-prev:hover{right:1px;left:auto}.ui-datepicker-rtl .ui-datepicker-next:hover{left:1px;right:auto}.ui-datepicker-rtl .ui-datepicker-buttonpane{clear:right}.ui-datepicker-rtl .ui-datepicker-buttonpane button{float:left}.ui-datepicker-rtl .ui-datepicker-buttonpane button.ui-datepicker-current{float:right}.ui-datepicker-rtl .ui-datepicker-group{float:right}.ui-datepicker-rtl .ui-datepicker-group-last .ui-datepicker-header{border-right-width:0;border-left-width:1px}.ui-datepicker-rtl .ui-datepicker-group-middle .ui-datepicker-header{border-right-width:0;border-left-width:1px}.ui-datepicker-cover{position:absolute;z-index:-1;filter:mask();top:-4px;left:-4px;width:200px;height:200px}.ui-dialog{position:absolute;padding:.2em;width:300px;overflow:hidden}.ui-dialog .ui-dialog-titlebar{padding:.4em 1em;position:relative}.ui-dialog .ui-dialog-title{float:left;margin:.1em 16px .1em 0}.ui-dialog .ui-dialog-titlebar-close{position:absolute;right:.3em;top:50%;width:19px;margin:-10px 0 0 0;padding:1px;height:18px}.ui-dialog .ui-dialog-titlebar-close span{display:block;margin:1px}.ui-dialog .ui-dialog-titlebar-close:hover,.ui-dialog .ui-dialog-titlebar-close:focus{padding:0}.ui-dialog .ui-dialog-content{position:relative;border:0;padding:.5em 1em;background:none;overflow:auto;zoom:1}.ui-dialog .ui-dialog-buttonpane{text-align:left;border-width:1px 0 0 0;background-image:none;margin:.5em 0 0 0;padding:.3em 1em .5em .4em}.ui-dialog .ui-dialog-buttonpane .ui-dialog-buttonset{float:right}.ui-dialog .ui-dialog-buttonpane button{margin:.5em .4em .5em 0;cursor:pointer}.ui-dialog .ui-resizable-se{width:14px;height:14px;right:3px;bottom:3px}.ui-draggable .ui-dialog-titlebar{cursor:move}.ui-progressbar{height:2em;text-align:left;overflow:hidden}.ui-progressbar .ui-progressbar-value{margin:-1px;height:100%}.ui-resizable{position:relative}.ui-resizable-handle{position:absolute;font-size:0.1px;display:block}.ui-resizable-disabled .ui-resizable-handle,.ui-resizable-autohide .ui-resizable-handle{display:none}.ui-resizable-n{cursor:n-resize;height:7px;width:100%;top:-5px;left:0}.ui-resizable-s{cursor:s-resize;height:7px;width:100%;bottom:-5px;left:0}.ui-resizable-e{cursor:e-resize;width:7px;right:-5px;top:0;height:100%}.ui-resizable-w{cursor:w-resize;width:7px;left:-5px;top:0;height:100%}.ui-resizable-se{cursor:se-resize;width:12px;height:12px;right:1px;bottom:1px}.ui-resizable-sw{cursor:sw-resize;width:9px;height:9px;left:-5px;bottom:-5px}.ui-resizable-nw{cursor:nw-resize;width:9px;height:9px;left:-5px;top:-5px}.ui-resizable-ne{cursor:ne-resize;width:9px;height:9px;right:-5px;top:-5px}.ui-selectable-helper{position:absolute;z-index:100;border:1px dotted black}.ui-slider{position:relative;text-align:left}.ui-slider .ui-slider-handle{position:absolute;z-index:2;width:1.2em;height:1.2em;cursor:default}.ui-slider .ui-slider-range{position:absolute;z-index:1;font-size:.7em;display:block;border:0;background-position:0 0}.ui-slider-horizontal{height:.8em}.ui-slider-horizontal .ui-slider-handle{top:-.3em;margin-left:-.6em}.ui-slider-horizontal .ui-slider-range{top:0;height:100%}.ui-slider-horizontal .ui-slider-range-min{left:0}.ui-slider-horizontal .ui-slider-range-max{right:0}.ui-slider-vertical{width:.8em;height:100px}.ui-slider-vertical .ui-slider-handle{left:-.3em;margin-left:0;margin-bottom:-.6em}.ui-slider-vertical .ui-slider-range{left:0;width:100%}.ui-slider-vertical .ui-slider-range-min{bottom:0}.ui-slider-vertical .ui-slider-range-max{top:0}.ui-tabs{position:relative;padding:.2em;zoom:1}.ui-tabs .ui-tabs-nav{margin:0;padding:.2em .2em 0}.ui-tabs .ui-tabs-nav li{list-style:none;float:left;position:relative;top:1px;margin:0 .2em 1px 0;border-bottom:0!important;padding:0;white-space:nowrap}.ui-tabs .ui-tabs-nav li a{float:left;padding:.5em 1em;text-decoration:none}.ui-tabs .ui-tabs-nav li.ui-tabs-selected{margin-bottom:0;padding-bottom:1px}.ui-tabs .ui-tabs-nav li.ui-tabs-selected a,.ui-tabs .ui-tabs-nav li.ui-state-disabled a,.ui-tabs .ui-tabs-nav li.ui-state-processing a{cursor:text}.ui-tabs .ui-tabs-nav li a,.ui-tabs.ui-tabs-collapsible .ui-tabs-nav li.ui-tabs-selected a{cursor:pointer}.ui-tabs .ui-tabs-panel{display:block;border-width:0;padding:1em 1.4em;background:none}.ui-tabs .ui-tabs-hide{display:none!important}.ui-widget{font-family:Verdana,Arial,sans-serif;font-size:1.1em}.ui-widget .ui-widget{font-size:1em}.ui-widget input,.ui-widget select,.ui-widget textarea,.ui-widget button{font-family:Verdana,Arial,sans-serif;font-size:1em}.ui-widget-content{border:1px solid #aaa;background:#fff url("../../lib/css/images/ui-bg_flat_75_ffffff_40x100.8692e6efddf8.png") 50% 50% repeat-x;color:#222}.ui-widget-content a{color:#222}.ui-widget-header{border:1px solid #aaa;background:#ccc url("../../lib/css/images/ui-bg_highlight-soft_75_cccccc_1x100.72c593d16e99.png") 50% 50% repeat-x;color:#222;font-weight:bold}.ui-widget-header a{color:#222}.ui-state-default,.ui-widget-content .ui-state-default,.ui-widget-header .ui-state-default{border:1px solid #d3d3d3;background:#e6e6e6 url("../../lib/css/images/ui-bg_glass_75_e6e6e6_1x400.f4254356c2a8.png") 50% 50% repeat-x;font-weight:normal;color:#555}.ui-state-default a,.ui-state-default a:link,.ui-state-default a:visited{color:#555;text-decoration:none}.ui-state-hover,.ui-widget-content .ui-state-hover,.ui-widget-header .ui-state-hover,.ui-state-focus,.ui-widget-content .ui-state-focus,.ui-widget-header .ui-state-focus{border:1px solid #999;background:#dadada url("../../lib/css/images/ui-bg_glass_75_dadada_1x400.c12c6510dad3.png") 50% 50% repeat-x;font-weight:normal;color:#212121}.ui-state-hover a,.ui-state-hover a:hover{color:#212121;text-decoration:none}.ui-state-active,.ui-widget-content .ui-state-active,.ui-widget-header .ui-state-active{border:1px solid #aaa;background:#fff url("../../lib/css/images/ui-bg_glass_65_ffffff_1x400.e5a8f32e28fd.png") 50% 50% repeat-x;font-weight:normal;color:#212121}.ui-state-active a,.ui-state-active a:link,.ui-state-active a:visited{color:#212121;text-decoration:none}.ui-widget:active{outline:none}.ui-state-highlight,.ui-widget-content .ui-state-highlight,.ui-widget-header .ui-state-highlight{border:1px solid #fcefa1;background:#fbf9ee url("../../lib/css/images/ui-bg_glass_55_fbf9ee_1x400.f8f4558e0b92.png") 50% 50% repeat-x;color:#363636}.ui-state-highlight a,.ui-widget-content .ui-state-highlight a,.ui-widget-header .ui-state-highlight a{color:#363636}.ui-state-error,.ui-widget-content .ui-state-error,.ui-widget-header .ui-state-error{border:1px solid #cd0a0a;background:#fef1ec url("../../lib/css/images/ui-bg_glass_95_fef1ec_1x400.5a3be2d8fff8.png") 50% 50% repeat-x;color:#cd0a0a}.ui-state-error a,.ui-widget-content .ui-state-error a,.ui-widget-header .ui-state-error a{color:#cd0a0a}.ui-state-error-text,.ui-widget-content .ui-state-error-text,.ui-widget-header .ui-state-error-text{color:#cd0a0a}.ui-priority-primary,.ui-widget-content .ui-priority-primary,.ui-widget-header .ui-priority-primary{font-weight:bold}.ui-priority-secondary,.ui-widget-content .ui-priority-secondary,.ui-widget-header .ui-priority-secondary{opacity:.7;filter:Alpha(Opacity=70);font-weight:normal}.ui-state-disabled,.ui-widget-content .ui-state-disabled,.ui-widget-header .ui-state-disabled{opacity:.35;filter:Alpha(Opacity=35);background-image:none}.ui-icon{width:16px;height:16px;background-image:url("../../lib/css/images/ui-icons_222222_256x240.9129e086dc48.png")}.ui-widget-content .ui-icon{background-image:url("../../lib/css/images/ui-icons_222222_256x240.9129e086dc48.png")}.ui-widget-header .ui-icon{background-image:url("../../lib/css/images/ui-icons_222222_256x240.9129e086dc48.png")}.ui-state-default .ui-icon{background-image:url("../../lib/css/images/ui-icons_888888_256x240.faf6f5dc44e7.png")}.ui-state-hover .ui-icon,.ui-state-focus .ui-icon{background-image:url("../../lib/css/images/ui-icons_454545_256x240.771099482bdc.png")}.ui-state-active .ui-icon{background-image:url("../../lib/css/images/ui-icons_454545_256x240.771099482bdc.png")}.ui-state-highlight .ui-icon{background-image:url("../../lib/css/images/ui-icons_2e83ff_256x240.25162bf857a8.png")}.ui-state-error .ui-icon,.ui-state-error-text .ui-icon{background-image:url("../../lib/css/images/ui-icons_cd0a0a_256x240.5d8808d43cef.png")}.ui-icon-carat-1-n{background-position:0 0}.ui-icon-carat-1-ne{background-position:-16px 0}.ui-icon-carat-1-e{background-position:-32px 0}.ui-icon-carat-1-se{background-position:-48px 0}.ui-icon-carat-1-s{background-position:-64px 0}.ui-icon-carat-1-sw{background-position:-80px 0}.ui-icon-carat-1-w{background-position:-96px 0}.ui-icon-carat-1-nw{background-position:-112px 0}.ui-icon-carat-2-n-s{background-position:-128px 0}.ui-icon-carat-2-e-w{background-position:-144px 0}.ui-icon-triangle-1-n{background-position:0 -16px}.ui-icon-triangle-1-ne{background-position:-16px -16px}.ui-icon-triangle-1-e{background-position:-32px -16px}.ui-icon-triangle-1-se{background-position:-48px -16px}.ui-icon-triangle-1-s{background-position:-64px -16px}.ui-icon-triangle-1-sw{background-position:-80px -16px}.ui-icon-triangle-1-w{background-position:-96px -16px}.ui-icon-triangle-1-nw{background-position:-112px -16px}.ui-icon-triangle-2-n-s{background-position:-128px -16px}.ui-icon-triangle-2-e-w{background-position:-144px -16px}.ui-icon-arrow-1-n{background-position:0 -32px}.ui-icon-arrow-1-ne{background-position:-16px -32px}.ui-icon-arrow-1-e{background-position:-32px -32px}.ui-icon-arrow-1-se{background-position:-48px -32px}.ui-icon-arrow-1-s{background-position:-64px -32px}.ui-icon-arrow-1-sw{background-position:-80px -32px}.ui-icon-arrow-1-w{background-position:-96px -32px}.ui-icon-arrow-1-nw{background-position:-112px -32px}.ui-icon-arrow-2-n-s{background-position:-128px -32px}.ui-icon-arrow-2-ne-sw{background-position:-144px -32px}.ui-icon-arrow-2-e-w{background-position:-160px -32px}.ui-icon-arrow-2-se-nw{background-position:-176px -32px}.ui-icon-arrowstop-1-n{background-position:-192px -32px}.ui-icon-arrowstop-1-e{background-position:-208px -32px}.ui-icon-arrowstop-1-s{background-position:-224px -32px}.ui-icon-arrowstop-1-w{background-position:-240px -32px}.ui-icon-arrowthick-1-n{background-position:0 -48px}.ui-icon-arrowthick-1-ne{background-position:-16px -48px}.ui-icon-arrowthick-1-e{background-position:-32px -48px}.ui-icon-arrowthick-1-se{background-position:-48px -48px}.ui-icon-arrowthick-1-s{background-position:-64px -48px}.ui-icon-arrowthick-1-sw{background-position:-80px -48px}.ui-icon-arrowthick-1-w{background-position:-96px -48px}.ui-icon-arrowthick-1-nw{background-position:-112px -48px}.ui-icon-arrowthick-2-n-s{background-position:-128px -48px}.ui-icon-arrowthick-2-ne-sw{background-position:-144px -48px}.ui-icon-arrowthick-2-e-w{background-position:-160px -48px}.ui-icon-arrowthick-2-se-nw{background-position:-176px -48px}.ui-icon-arrowthickstop-1-n{background-position:-192px -48px}.ui-icon-arrowthickstop-1-e{background-position:-208px -48px}.ui-icon-arrowthickstop-1-s{background-position:-224px -48px}.ui-icon-arrowthickstop-1-w{background-position:-240px -48px}.ui-icon-arrowreturnthick-1-w{background-position:0 -64px}.ui-icon-arrowreturnthick-1-n{background-position:-16px -64px}.ui-icon-arrowreturnthick-1-e{background-position:-32px -64px}.ui-icon-arrowreturnthick-1-s{background-position:-48px -64px}.ui-icon-arrowreturn-1-w{background-position:-64px -64px}.ui-icon-arrowreturn-1-n{background-position:-80px -64px}.ui-icon-arrowreturn-1-e{background-position:-96px -64px}.ui-icon-arrowreturn-1-s{background-position:-112px -64px}.ui-icon-arrowrefresh-1-w{background-position:-128px -64px}.ui-icon-arrowrefresh-1-n{background-position:-144px -64px}.ui-icon-arrowrefresh-1-e{background-position:-160px -64px}.ui-icon-arrowrefresh-1-s{background-position:-176px -64px}.ui-icon-arrow-4{background-position:0 -80px}.ui-icon-arrow-4-diag{background-position:-16px -80px}.ui-icon-extlink{background-position:-32px -80px}.ui-icon-newwin{background-position:-48px -80px}.ui-icon-refresh{background-position:-64px -80px}.ui-icon-shuffle{background-position:-80px -80px}.ui-icon-transfer-e-w{background-position:-96px -80px}.ui-icon-transferthick-e-w{background-position:-112px -80px}.ui-icon-folder-collapsed{background-position:0 -96px}.ui-icon-folder-open{background-position:-16px -96px}.ui-icon-document{background-position:-32px -96px}.ui-icon-document-b{background-position:-48px -96px}.ui-icon-note{background-position:-64px -96px}.ui-icon-mail-closed{background-position:-80px -96px}.ui-icon-mail-open{background-position:-96px -96px}.ui-icon-suitcase{background-position:-112px -96px}.ui-icon-comment{background-position:-128px -96px}.ui-icon-person{background-position:-144px -96px}.ui-icon-print{background-position:-160px -96px}.ui-icon-trash{background-position:-176px -96px}.ui-icon-locked{background-position:-192px -96px}.ui-icon-unlocked{background-position:-208px -96px}.ui-icon-bookmark{background-position:-224px -96px}.ui-icon-tag{background-position:-240px -96px}.ui-icon-home{background-position:0 -112px}.ui-icon-flag{background-position:-16px -112px}.ui-icon-calendar{background-position:-32px -112px}.ui-icon-cart{background-position:-48px -112px}.ui-icon-pencil{background-position:-64px -112px}.ui-icon-clock{background-position:-80px -112px}.ui-icon-disk{background-position:-96px -112px}.ui-icon-calculator{background-position:-112px -112px}.ui-icon-zoomin{background-position:-128px -112px}.ui-icon-zoomout{background-position:-144px -112px}.ui-icon-search{background-position:-160px -112px}.ui-icon-wrench{background-position:-176px -112px}.ui-icon-gear{background-position:-192px -112px}.ui-icon-heart{background-position:-208px -112px}.ui-icon-star{background-position:-224px -112px}.ui-icon-link{background-position:-240px -112px}.ui-icon-cancel{background-position:0 -128px}.ui-icon-plus{background-position:-16px -128px}.ui-icon-plusthick{background-position:-32px -128px}.ui-icon-minus{background-position:-48px -128px}.ui-icon-minusthick{background-position:-64px -128px}.ui-icon-close{background-position:-80px -128px}.ui-icon-closethick{background-position:-96px -128px}.ui-icon-key{background-position:-112px -128px}.ui-icon-lightbulb{background-position:-128px -128px}.ui-icon-scissors{background-position:-144px -128px}.ui-icon-clipboard{background-position:-160px -128px}.ui-icon-copy{background-position:-176px -128px}.ui-icon-contact{background-position:-192px -128px}.ui-icon-image{background-position:-208px -128px}.ui-icon-video{background-position:-224px -128px}.ui-icon-script{background-position:-240px -128px}.ui-icon-alert{background-position:0 -144px}.ui-icon-info{background-position:-16px -144px}.ui-icon-notice{background-position:-32px -144px}.ui-icon-help{background-position:-48px -144px}.ui-icon-check{background-position:-64px -144px}.ui-icon-bullet{background-position:-80px -144px}.ui-icon-radio-off{background-position:-96px -144px}.ui-icon-radio-on{background-position:-112px -144px}.ui-icon-pin-w{background-position:-128px -144px}.ui-icon-pin-s{background-position:-144px -144px}.ui-icon-play{background-position:0 -160px}.ui-icon-pause{background-position:-16px -160px}.ui-icon-seek-next{background-position:-32px -160px}.ui-icon-seek-prev{background-position:-48px -160px}.ui-icon-seek-end{background-position:-64px -160px}.ui-icon-seek-start{background-position:-80px -160px}.ui-icon-seek-first{background-position:-80px -160px}.ui-icon-stop{background-position:-96px -160px}.ui-icon-eject{background-position:-112px -160px}.ui-icon-volume-off{background-position:-128px -160px}.ui-icon-volume-on{background-position:-144px -160px}.ui-icon-power{background-position:0 -176px}.ui-icon-signal-diag{background-position:-16px -176px}.ui-icon-signal{background-position:-32px -176px}.ui-icon-battery-0{background-position:-48px -176px}.ui-icon-battery-1{background-position:-64px -176px}.ui-icon-battery-2{background-position:-80px -176px}.ui-icon-battery-3{background-position:-96px -176px}.ui-icon-circle-plus{background-position:0 -192px}.ui-icon-circle-minus{background-position:-16px -192px}.ui-icon-circle-close{background-position:-32px -192px}.ui-icon-circle-triangle-e{background-position:-48px -192px}.ui-icon-circle-triangle-s{background-position:-64px -192px}.ui-icon-circle-triangle-w{background-position:-80px -192px}.ui-icon-circle-triangle-n{background-position:-96px -192px}.ui-icon-circle-arrow-e{background-position:-112px -192px}.ui-icon-circle-arrow-s{background-position:-128px -192px}.ui-icon-circle-arrow-w{background-position:-144px -192px}.ui-icon-circle-arrow-n{background-position:-160px -192px}.ui-icon-circle-zoomin{background-position:-176px -192px}.ui-icon-circle-zoomout{background-position:-192px -192px}.ui-icon-circle-check{background-position:-208px -192px}.ui-icon-circlesmall-plus{background-position:0 -208px}.ui-icon-circlesmall-minus{background-position:-16px -208px}.ui-icon-circlesmall-close{background-position:-32px -208px}.ui-icon-squaresmall-plus{background-position:-48px -208px}.ui-icon-squaresmall-minus{background-position:-64px -208px}.ui-icon-squaresmall-close{background-position:-80px -208px}.ui-icon-grip-dotted-vertical{background-position:0 -224px}.ui-icon-grip-dotted-horizontal{background-position:-16px -224px}.ui-icon-grip-solid-vertical{background-position:-32px -224px}.ui-icon-grip-solid-horizontal{background-position:-48px -224px}.ui-icon-gripsmall-diagonal-se{background-position:-64px -224px}.ui-icon-grip-diagonal-se{background-position:-80px -224px}.ui-corner-all,.ui-corner-top,.ui-corner-left,.ui-corner-tl{-moz-border-radius-topleft:4px;-webkit-border-top-left-radius:4px;-khtml-border-top-left-radius:4px;border-top-left-radius:4px}.ui-corner-all,.ui-corner-top,.ui-corner-right,.ui-corner-tr{-moz-border-radius-topright:4px;-webkit-border-top-right-radius:4px;-khtml-border-top-right-radius:4px;border-top-right-radius:4px}.ui-corner-all,.ui-corner-bottom,.ui-corner-left,.ui-corner-bl{-moz-border-radius-bottomleft:4px;-webkit-border-bottom-left-radius:4px;-khtml-border-bottom-left-radius:4px;border-bottom-left-radius:4px}.ui-corner-all,.ui-corner-bottom,.ui-corner-right,.ui-corner-br{-moz-border-radius-bottomright:4px;-webkit-border-bottom-right-radius:4px;-khtml-border-bottom-right-radius:4px;border-bottom-right-radius:4px}.ui-widget-overlay{background:#aaa url("../../lib/css/images/ui-bg_flat_0_aaaaaa_40x100.2a44fbdb7360.png") 50% 50% repeat-x;opacity:.3;filter:Alpha(Opacity=30)}.ui-widget-shadow{margin:-8px 0 0 -8px;padding:8px;background:#aaa url("../../lib/css/images/ui-bg_flat_0_aaaaaa_40x100.2a44fbdb7360.png") 50% 50% repeat-x;opacity:.3;filter:Alpha(Opacity=30);-moz-border-radius:8px;-khtml-border-radius:8px;-webkit-border-radius:8px;border-radius:8px}
/* RB Colors */
/*
 * At least on Chrome, using points instead of pixels does not reliably
 * result in identical sizings when there are elements positioned inside
 * a <pre>. Pixels fixes this.
 */
/* Diff styles */
/* !(*%!(&^ (see http://www.positioniseverything.net/easyclearing.html) */
.clearfix {
  display: inline-block;
}
.clearfix:after {
  clear: both;
  content: ".";
  display: block;
  height: 0;
  visibility: hidden;
}
/* Hides from IE-mac \*/
* html .clearfix {
  height: 1%;
}
.clearfix {
  display: block;
}
/* End hide from IE-mac */
body {
  background-color: white;
  color: black;
  font-family: Verdana, Arial, Helvetica, Sans-Serif;
  font-size: 8pt;
  margin: 0;
  padding: 0;
}
form th {
  text-align: right;
  vertical-align: top;
}
h1 {
  font-size: 10pt;
}
h3 {
  margin-top: 2em;
}
img {
  border: 0;
}
label.required:after {
  content: ' *';
  font-weight: bold;
  font-size: 10px;
  color: darkred;
}
pre,
textarea,
.CodeMirror,
.editable.field-text-area {
  font-size: 13px;
  font-family: monospace;
  line-height: 15px;
}
span.label-draft {
  color: #008000;
}
span.label-submitted {
  color: #b7a93e;
}
span.label-discarded {
  color: #fe5757;
}
td.help {
  font-size: 10px;
  color: #333;
  padding-top: 0;
}
textarea {
  resize: none;
}
ul.errorlist {
  color: #DD0000;
  display: inline;
  font-weight: bold;
  margin: 0 0 0 15px;
  padding: 4px 8px;
}
ul.errorlist li {
  display: inline;
}
.loading img {
  margin-right: 4px;
  vertical-align: text-bottom;
}
/****************************************************************************
 * Buttons
 ****************************************************************************/
.btn,
input[type="button"],
input[type="submit"] {
  cursor: pointer;
  display: inline-block;
  background-color: #e6e6e6;
  background-repeat: no-repeat;
  background-image: -webkit-gradient(linear, 0 0, 0 100%, from(#ffffff), color-stop(25%, #ffffff), to(#e6e6e6));
  background-image: -webkit-linear-gradient(#ffffff, #ffffff 25%, #e6e6e6);
  background-image: -moz-linear-gradient(top, #ffffff, #ffffff 25%, #e6e6e6);
  background-image: -ms-linear-gradient(#ffffff, #ffffff 25%, #e6e6e6);
  background-image: -o-linear-gradient(#ffffff, #ffffff 25%, #e6e6e6);
  background-image: linear-gradient(#ffffff, #ffffff 25%, #e6e6e6);
  filter: progid:DXImageTransform.Microsoft.gradient(startColorstr='#ffffff', endColorstr='#e6e6e6', GradientType=0);
  padding: 3px 8px;
  text-shadow: 0 1px 1px rgba(255, 255, 255, 0.75);
  color: #333;
  font-size: 13px;
  line-height: normal;
  border: 1px solid #ccc;
  border-bottom-color: #bbb;
  -moz-border-radius: 4px;
  -webkit-border-radius: 4px;
  border-radius: 4px;
  -moz-box-shadow: inset 0 1px 0 rgba(255, 255, 255, 0.2) 0 1px 2px rgba(0, 0, 0, 0.05);
  -webkit-box-shadow: inset 0 1px 0 rgba(255, 255, 255, 0.2) 0 1px 2px rgba(0, 0, 0, 0.05);
  box-shadow: inset 0 1px 0 rgba(255, 255, 255, 0.2) 0 1px 2px rgba(0, 0, 0, 0.05);
  -webkit-transition: 0.1s linear all;
  -moz-transition: 0.1s linear all;
  transition: 0.1s linear all;
}
.btn:hover,
input[type="button"]:hover,
input[type="submit"]:hover {
  background-position: 0 -15px;
  color: #333;
  text-decoration: none;
}
.btn.primary,
input[type="button"].primary,
input[type="submit"].primary,
.btn.default,
input[type="button"].default,
input[type="submit"].default {
  color: white;
  text-shadow: none;
  background-color: #44679a;
  background-repeat: repeat-x;
  background-image: -khtml-gradient(linear, left top, left bottom, from(#7e9bc6), to(#44679a));
  background-image: -moz-linear-gradient(top, #7e9bc6, #44679a);
  background-image: -ms-linear-gradient(top, #7e9bc6, #44679a);
  background-image: -webkit-gradient(linear, left top, left bottom, color-stop(0%, #7e9bc6), color-stop(100%, #44679a));
  background-image: -webkit-linear-gradient(top, #7e9bc6, #44679a);
  background-image: -o-linear-gradient(top, #7e9bc6, #44679a);
  background-image: linear-gradient(top, #7e9bc6, #44679a);
  filter: progid:DXImageTransform.Microsoft.gradient(startColorstr='#7e9bc6', endColorstr='#44679a', GradientType=0);
  text-shadow: 0 -1px 0 rgba(0, 0, 0, 0.25);
  border-color: #44679a #44679a #2d4365;
  border-color: rgba(0, 0, 0, 0.1) rgba(0, 0, 0, 0.1) rgba(0, 0, 0, 0.25);
}
.btn:active,
input[type="button"]:active,
input[type="submit"]:active {
  -moz-box-shadow: inset 0 2px 4px rgba(0, 0, 0, 0.25) 0 1px 2px rgba(0, 0, 0, 0.05);
  -webkit-box-shadow: inset 0 2px 4px rgba(0, 0, 0, 0.25) 0 1px 2px rgba(0, 0, 0, 0.05);
  box-shadow: inset 0 2px 4px rgba(0, 0, 0, 0.25) 0 1px 2px rgba(0, 0, 0, 0.05);
}
.btn.disabled,
input[type="button"].disabled,
input[type="submit"].disabled {
  cursor: default;
  background-image: none;
  filter: progid:DXImageTransform.Microsoft.gradient(enabled = false);
  opacity: 0.65;
  filter: alpha(opacity=65);
  -moz-opacity: 0.65;
  -moz-box-shadow: none;
  -webkit-box-shadow: none;
  box-shadow: none;
}
.btn[disabled],
input[type="button"][disabled],
input[type="submit"][disabled] {
  cursor: default;
  background-image: none;
  filter: progid:DXImageTransform.Microsoft.gradient(enabled = false);
  opacity: 0.65;
  filter: alpha(opacity=65);
  -moz-opacity: 0.65;
  -moz-box-shadow: none;
  -webkit-box-shadow: none;
  box-shadow: none;
}
.btn.large,
input[type="button"].large,
input[type="submit"].large {
  font-size: 130%;
  line-height: normal;
  padding: 9px 14px 9px;
  -moz-border-radius: 6px;
  -webkit-border-radius: 6px;
  border-radius: 6px;
}
.btn.small,
input[type="button"].small,
input[type="submit"].small {
  padding: 7px 9px 7px;
  font-size: 11px;
}
.btn.danger,
input[type="button"].danger,
input[type="submit"].danger {
  color: white !important;
  background-color: #c43c35;
  background-repeat: repeat-x;
  background-image: -khtml-gradient(linear, left top, left bottom, from(#ee5f5b), to(#c43c35));
  background-image: -moz-linear-gradient(top, #ee5f5b, #c43c35);
  background-image: -ms-linear-gradient(top, #ee5f5b, #c43c35);
  background-image: -webkit-gradient(linear, left top, left bottom, color-stop(0%, #ee5f5b), color-stop(100%, #c43c35));
  background-image: -webkit-linear-gradient(top, #ee5f5b, #c43c35);
  background-image: -o-linear-gradient(top, #ee5f5b, #c43c35);
  background-image: linear-gradient(top, #ee5f5b, #c43c35);
  filter: progid:DXImageTransform.Microsoft.gradient(startColorstr='#ee5f5b', endColorstr='#c43c35', GradientType=0);
  text-shadow: 0 -1px 0 rgba(0, 0, 0, 0.25);
  border-color: #c43c35 #c43c35 #882a25;
  border-color: rgba(0, 0, 0, 0.1) rgba(0, 0, 0, 0.1) rgba(0, 0, 0, 0.25);
}
button.btn::-moz-focus-inner,
input[type=submit].btn::-moz-focus-inner {
  padding: 0;
  border: 0;
}
/****************************************************************************
 * Banners
 ****************************************************************************/
.banner {
  background: #cdff9c;
  border: 1px #4d9c00 solid;
  margin-bottom: 10px;
  padding: 8px 10px;
  z-index: 90;
}
.banner > h1 {
  display: inline;
  margin-right: 0.2em;
  padding: 0;
}
.banner > input {
  margin-left: 10px;
}
/****************************************************************************
 * Box classes
 ****************************************************************************/
.box,
.modalbox {
  background-color: #ffffff;
  background-image: url("../images/box_top_bg.eea86c350dcf.png");
  background-position: top left;
  background-repeat: repeat-x;
  border: 1px #888a85 solid;
  -moz-border-radius: 6px;
  -webkit-border-radius: 6px;
  border-radius: 6px;
  -moz-box-shadow: 1px 1px 2px rgba(0, 0, 0, 0.15);
  -webkit-box-shadow: 1px 1px 2px rgba(0, 0, 0, 0.15);
  box-shadow: 1px 1px 2px rgba(0, 0, 0, 0.15);
}
.box.important {
  background-color: #fdf2a0;
  background-image: url("../images/gold_box_top_bg.33df1a7c2e1b.png");
  border: 1px #b7a93e solid;
  padding: 0 10px;
}
.box.important .box-inner {
  background: url("../images/important.5e4ebe0a938f.png") no-repeat top left;
  padding: 0 0 0 50px;
}
.box.tip {
  background-color: #fdf2a0;
  background-image: url("../images/gold_box_top_bg.33df1a7c2e1b.png");
  border: 1px #b7a93e solid;
  padding: 0 10px;
}
.box.tip .box-inner {
  background: transparent;
}
.box.tip h1 {
  padding-left: 0;
}
.box.yay {
  background-color: #fdf2a0;
  background-image: url("../images/gold_box_top_bg.33df1a7c2e1b.png");
  border: 1px #b7a93e solid;
  padding: 0 10px;
}
.box.yay .box-inner {
  background: transparent;
  position: relative;
}
.box.yay h1 {
  padding-left: 2.5em;
}
.box.yay img {
  float: left;
  left: -5px;
  margin: 0;
  padding: 0;
  position: absolute;
  top: -20px;
}
.box .star {
  cursor: pointer;
}
.admin h1.title,
.box-head,
.modalbox-title {
  background: #c6dcf3;
  border-bottom: 1px #728eac solid;
  margin: 0;
  padding: 0.4em 0.6em;
  -moz-border-radius: 6px 6px 0 0;
  -webkit-border-radius: 6px 6px 0 0;
  border-radius: 6px 6px 0 0;
}
.box-title,
.modalbox-title {
  font-size: 120%;
}
.box-title {
  margin: 0;
  padding: 0;
}
.box-main {
  padding: 5px;
}
.box-container {
  margin-bottom: 10px;
  width: 100%;
}
.box-inner,
.modalbox-inner {
  background-image: url("../images/box_bottom_bg.62df21d4a671.png");
  background-position: bottom left;
  background-repeat: repeat-x;
  -moz-border-radius: 6px;
  -webkit-border-radius: 6px;
  border-radius: 6px;
  padding-bottom: 1px\9;
  /* IE wants this. The \9 targets IE 7,8,9. */
}
.box-inner form {
  margin: 0;
}
.modalbox,
.modalbox-bg {
  z-index: 100;
}
.datagrid-title,
.datagrid-titlebox {
  background: #c6dcf3 !important;
  -moz-border-radius: 6px 6px 0 0;
  -webkit-border-radius: 6px 6px 0 0;
  border-radius: 6px 6px 0 0;
}
.datagrid-main {
  -moz-border-radius: 6px;
  -webkit-border-radius: 6px;
  border-radius: 6px;
}
.datagrid-header {
  border-top: 0 !important;
}
.datagrid-header:last-child {
  border-right: 0;
}
.datagrid-menu {
  border: 1px #888A85 solid;
}
.datagrid tr:last-child td:first-child {
  -moz-border-radius: 0 0 0 6px;
  -webkit-border-radius: 0 0 0 6px;
  border-radius: 0 0 0 6px;
}
.datagrid tr:last-child td:last-child {
  -moz-border-radius: 0 0 6px 0;
  -webkit-border-radius: 0 0 6px 0;
  border-radius: 0 0 6px 0;
}
.datagrid td:last-child {
  border-right: 0;
}
.datagrid .issue-count,
.datagrid .shipit-count {
  display: inline-block;
  font-weight: bold;
  padding: 0.05em 0.8em;
  white-space: nowrap;
  -moz-border-radius: 10px;
  -webkit-border-radius: 10px;
  border-radius: 10px;
}
.datagrid .issue-count {
  border: 1px #9c8b00 solid;
  color: #205003;
  background-color: #ffe303;
  background-repeat: no-repeat;
  background-image: -webkit-gradient(linear, 0 0, 0 100%, from(#fff49c), color-stop(25%, #fff49c), to(#ffe303));
  background-image: -webkit-linear-gradient(#fff49c, #fff49c 25%, #ffe303);
  background-image: -moz-linear-gradient(top, #fff49c, #fff49c 25%, #ffe303);
  background-image: -ms-linear-gradient(#fff49c, #fff49c 25%, #ffe303);
  background-image: -o-linear-gradient(#fff49c, #fff49c 25%, #ffe303);
  background-image: linear-gradient(#fff49c, #fff49c 25%, #ffe303);
  filter: progid:DXImageTransform.Microsoft.gradient(startColorstr='#fff49c', endColorstr='#ffe303', GradientType=0);
}
.datagrid .issue-count .issue-icon {
  color: #695d00;
  display: inline-block;
  padding: 0 3px 0 1px;
  text-align: center;
}
.datagrid .shipit-count {
  background-image: url("../images/shipit_bg.a368d65cde1a.png");
  background-position: top left;
  background-repeat: repeat-x;
  background-color: #6bc810;
  border: 1px #478a06 solid;
  color: #205003;
}
.server-error-box .response-data {
  margin-top: 2em;
}
/****************************************************************************
 * Modal Boxes
 ****************************************************************************/
.modalbox {
  margin: 10px;
}
.modalbox .modalbox-contents {
  margin: 10px;
  position: relative;
  /* Makes this the offsetParent for calculations. */
}
.modalbox .modalbox-buttons {
  position: absolute;
  margin: 10px;
  text-align: right;
  bottom: 0;
  right: 0;
}
.modalbox .modalbox-buttons input {
  margin-left: 10px;
}
/****************************************************************************
 * Inline editor forms
 ****************************************************************************/
.editicon {
  margin-left: 4px;
}
.editicon .rb-icon {
  vertical-align: bottom;
}
.editable,
.editicon {
  line-height: 14px;
}
.inline-editor-form textarea,
.text-editor textarea {
  border: 1px #b8b5a0 solid;
  margin: 0;
  outline: none;
  padding: 10px;
  -moz-box-sizing: border-box;
  -webkit-box-sizing: border-box;
  box-sizing: border-box;
  /*
     * This prevents extra spacing below a text area in different browsers.
     * See http://stackoverflow.com/questions/7144843/extra-space-under-textarea-differs-along-browsers
     */
  vertical-align: top;
}
.inline-editor-form {
  display: block;
  margin: 0;
  padding: 0;
  white-space: nowrap;
}
.inline-editor-form .buttons input[type='button'] {
  margin-left: 6px;
  margin-right: 0;
}
.inline-editor-form .buttons input[type='button']:first-child {
  margin-left: 0;
}
.inline-editor-form input[type="text"] {
  border: 1px #888a85 solid;
  padding: 1px 2px;
}
.inline-editor-form input[type] + .buttons {
  padding-left: 6px;
}
.inline-editor-form textarea {
  margin-top: 5px;
}
.inline-editor-form textarea + .buttons,
.inline-editor-form .text-editor + .buttons {
  margin-top: 6px;
  white-space: normal;
}
.inline-editor-form .enable-markdown {
  margin-left: 1em;
}
.inline-editor-form .enable-markdown label {
  color: black;
  font-size: 8pt !important;
  font-weight: normal;
}
.loading-indicator {
  display: inline;
}
.text-editor {
  margin: 5px 0 0 0;
}
.text-editor textarea,
.text-editor .CodeMirror {
  margin: 0;
}
/****************************************************************************
 * Forms
 ****************************************************************************/
form table td {
  padding: 8px 8px 4px 4px;
}
form table td.label {
  text-align: right;
  font-weight: bold;
}
.formdlg tr {
  padding-top: 4px;
}
.formdlg td.label {
  white-space: nowrap;
}
.formdlg .error {
  color: #DD0000;
  font-weight: bold;
  margin-bottom: 10px;
  padding: 4px 8px;
}
.formdlg .errorlist {
  display: block;
  list-style: none;
  margin: 0;
  padding: 0;
}
.formdlg .errorlist li {
  display: list-item;
  padding: 2px 4px;
}
.formdlg .spinner {
  float: left;
}
.formdlg .spinner img {
  vertical-align: top;
}
.formdlg .spinner h1 {
  display: inline;
  margin-left: 10px;
}
/****************************************************************************
 * Account page
 ****************************************************************************/
.prefsbuttons {
  padding: 1em;
  text-align: center;
}
.prefsgroup ul {
  list-style: none;
  padding-left: 0;
  margin: 5px;
}
.section {
  margin-bottom: 20px;
}
.section .title {
  background: #DDDDDD;
  border: 1px black solid;
  padding: 4px 8px;
}
.section .body {
  margin-left: 20px;
  padding: 10px;
}
#headerbar {
  background: #333333;
  border-bottom: 1px #DDDDDD solid;
  color: white;
  margin: 0;
  padding: 0.5em;
}
#headerbar br {
  clear: both;
  height: 0;
}
#headerbar #title {
  color: white;
  display: inline;
  font-weight: normal;
  font-size: 120%;
  vertical-align: middle;
  margin: 0;
  padding: 0;
}
#headerbar #title a {
  color: white;
  font-size: 120%;
  text-decoration: none;
}
#headerbar #title .version {
  color: #E0E0E0;
  font-weight: normal;
  font-size: 90%;
}
#headerbar #logo {
  position: absolute;
  top: -2px;
  left: 6px;
}
#headerbar #search {
  float: right;
  margin: 0 1em;
}
#headerbar #search input {
  border: 1px black solid;
  margin: 0;
  padding: 2px;
  -moz-border-radius: 6px;
  -webkit-border-radius: 6px;
  border-radius: 6px;
  background-color: #EEEEEE;
  font-size: 1.2em;
}
#rbinfo {
  float: left;
  position: absolute;
  left: 0;
  padding-top: 0.2em;
  padding-left: 72px;
}
#rbinfo img {
  display: inline;
  vertical-align: middle;
}
#accountnav {
  list-style: none;
  margin: 0;
  padding: 0;
  z-index: 1;
  float: right;
}
#accountnav li {
  display: block;
  float: right;
  margin: -0.5em 0;
  padding: 0;
  position: relative;
  white-space: nowrap;
}
#accountnav li:hover {
  background: #555555;
}
#accountnav li:hover ul {
  display: block;
  z-index: 110;
}
#accountnav li a {
  color: white;
  display: block;
  font-size: 120%;
  margin: 0;
  padding: 0 1em;
  text-decoration: none;
  height: 32px;
  vertical-align: middle;
  line-height: 32px;
}
#accountnav li a.user-nav-item {
  padding-left: 0.2em;
}
#accountnav li img {
  display: inline;
  vertical-align: top;
  margin: 0;
  padding: 0;
}
#accountnav li ul {
  background-color: #333333;
  display: none;
  margin: 0;
  min-width: 9em;
  padding: 0;
  position: absolute;
  right: 0;
  -moz-border-radius: 0 0 6px 6px;
  -webkit-border-radius: 0 0 6px 6px;
  border-radius: 0 0 6px 6px;
}
#accountnav li ul li {
  float: none;
  margin: 0;
  padding: 0;
  text-align: right;
}
#accountnav li ul li:last-child {
  -moz-border-radius: 0 0 6px 6px;
  -webkit-border-radius: 0 0 6px 6px;
  border-radius: 0 0 6px 6px;
}
#accountnav li ul li a {
  margin: 0;
}
#activity-indicator {
  position: fixed;
  top: 0;
  left: 50%;
  text-align: center;
  font-weight: bold;
  background-color: #fce94f;
  background-image: url("../images/spinner.73e57937304d.gif");
  background-position: 0.4em 0.4em;
  background-repeat: no-repeat;
  border: 1px #c4a000 solid;
  border-top: 0;
  margin-left: -3em;
  padding: 0.5em 0.6em 0.5em 2.2em;
  width: 6em;
  z-index: 140;
}
#activity-indicator.error {
  background-color: #ffcd9c;
  width: 28em;
  margin-left: -14em;
  background-image: none;
  padding-left: 0.6em;
}
#activity-indicator.error a {
  margin-left: 1em;
  color: #0000CC;
  text-decoration: none;
}
#error div {
  background: #ffcd9c;
  border: 1px #9a4306 solid;
  margin-bottom: 10px;
  padding: 8px 10px;
}
#error h1 {
  display: inline;
  margin: 0;
  padding: 0;
}
#error input[type='submit'] {
  margin-left: 10px;
}
#manual-updates {
  padding-top: 1em;
}
#manual-updates .box-main {
  padding: 1em;
}
#manual-updates .box-main h1,
#manual-updates .box-main h2 {
  margin-top: 2em;
}
#manual-updates p {
  font-size: 9pt;
}
#navbar-container {
  margin: 0 0 1em 0;
}
#navbar {
  background: #c6dcf3;
  border-bottom: 1px #3465a4 solid;
  list-style: none;
  margin: 0;
  min-height: 1.5em;
  padding: 0.5em;
  padding-left: 72px;
}
#navbar li {
  display: inline;
  height: 30px;
  margin: 0 1em 0 0;
}
#navbar a {
  color: #0000aa;
  font-size: 110%;
  text-decoration: none;
}
#navbar .newreviewrequest {
  text-align: right;
}
body.admin #navbar {
  background: url("../images/admin/yellow_bg_small.08a30cc592eb.png") repeat-x #f9f2c9;
  margin: 0;
}
body.admin #navbar a {
  color: black;
}
#page-container {
  padding: 0 1em 1em 1em;
}
#searchbar {
  position: absolute;
  right: 0;
  margin-top: -3px;
}
/****************************************************************************
 * Login/Register pages
 ****************************************************************************/
#auth_container {
  font-size: 120%;
  margin: 7em 0 0 0;
  text-align: center;
}
#auth_container input[type="text"],
#auth_container input[type="password"],
#auth_container input[type="email"] {
  border: 1px #aaa solid;
  font-size: inherit;
  margin: 0;
  padding: 0.5em;
  width: 16em;
  -moz-border-radius: 6px;
  -webkit-border-radius: 6px;
  border-radius: 6px;
  -moz-box-sizing: border-box;
  -webkit-box-sizing: border-box;
  box-sizing: border-box;
}
#auth_container .auth-button-container {
  width: 16em;
}
#auth_container .auth-button-container input {
  font-size: 120%;
  margin: 0;
  padding: 0.6em;
  width: 100%;
  -moz-box-sizing: border-box;
  -webkit-box-sizing: border-box;
  box-sizing: border-box;
}
#auth_container .auth-form-row {
  clear: both;
  margin: 1.5em 0;
  padding-left: 12em;
}
#auth_container .auth-form-row.auth-field-row {
  padding-left: 0;
}
#auth_container .auth-form-row.auth-field-row label {
  float: left;
  font-weight: normal;
  padding: 0.5em 1em 0.5em 0.5em;
  text-align: right;
  width: 12em;
  -moz-box-sizing: border-box;
  -webkit-box-sizing: border-box;
  box-sizing: border-box;
}
#auth_container .auth-form-row.auth-field-row .errorlist {
  display: block;
  margin: 0.5em 0 0 12em;
  width: 16em;
}
#auth_container .auth-form-row.auth-field-row .errorlist li {
  font-weight: normal;
  font-size: 9pt;
}
#auth_container .auth-header {
  margin: 0 0 1em 0;
}
#auth_container .auth-header h1 {
  font-size: 120%;
  margin: 1em 0;
  padding: 0;
}
#auth_container .auth-header p {
  color: #444;
  margin: 1em 0;
}
#auth_container .auth-header .errorbox {
  display: inline-block;
  text-align: center;
}
#auth_container .auth-header .errorbox .errorlist {
  text-align: left;
}
#auth_container .auth-section {
  display: inline-block;
  margin: 0 2em;
  text-align: left;
  vertical-align: top;
}
#auth_container .auth-section.main-auth-section {
  margin-left: -10em;
}
#auth_container .errorlist {
  margin: 0;
  padding: 0;
}
#auth_container .errorlist li {
  display: block;
  font-weight: normal;
  margin: 0 0 1em 0;
}
#auth_container #login_form .login-links {
  margin-top: 2em;
  text-align: center;
}
#auth_container #login_form .login-links p {
  width: 16em;
}
#auth_container #login_form .login-links p a {
  color: blue;
  text-decoration: none;
}
#auth_container #register_form .register-captcha-row {
  padding-left: 0;
}
#auth_container #register_form .register-captcha-row .register-captcha-container {
  float: right;
}
/****************************************************************************
 * Auto-complete widget
 ****************************************************************************/
.ui-autocomplete-results {
  background: #ffffff;
  border: 1px solid #808080;
  overflow: hidden;
  position: absolute;
  width: 100%;
  z-index: 110;
}
.ui-autocomplete-results ul {
  margin: 0;
  padding: 0;
  width: 100%;
}
.ui-autocomplete-results ul li {
  cursor: pointer;
  margin: 0;
  padding: 3px 6px;
  position: relative;
  white-space: nowrap;
}
.ui-autocomplete-results ul li span {
  margin-left: 6px;
  position: absolute;
  right: 6px;
}
.ui-autocomplete-results .ui-autocomplete-over {
  background: #71a5db;
  color: #FFF;
}
.ui-autocomplete-footer {
  background: #ECECEC;
  border-top: 1px #C0C0C0 solid;
  padding: 3px 6px;
}
/****************************************************************************
 * User page hover
 ****************************************************************************/
#user-infobox {
  background: #F9F9F9;
  border: 1px black solid;
  display: block;
  min-height: 12em;
  padding: 1em;
  position: absolute;
  min-width: 30em;
  z-index: 100;
  -moz-border-radius: 10px;
  -webkit-border-radius: 10px;
  border-radius: 10px;
  -moz-box-shadow: 0px 0px 4px #000000;
  -webkit-box-shadow: 0px 0px 4px #000000;
  box-shadow: 0px 0px 4px #000000;
}
#user-infobox.loading {
  background-image: url("../images/spinner.73e57937304d.gif");
  background-position: 1em 1em;
  background-repeat: no-repeat;
}
#user-infobox .infobox-pic {
  float: left;
  width: 85px;
  margin-right: 1em;
}
#user-infobox .infobox-text {
  color: black;
  float: left;
  max-width: 200px;
  word-wrap: break-word;
}
#user-infobox a {
  text-decoration: none;
}
#user-infobox a:hover {
  text-decoration: underline;
}
#user-infobox p {
  font-size: 1.2em;
  margin: 0.5em 0;
  padding: 0;
}
#user-infobox h2 {
  font-size: 1.4em;
  margin: 0 0 0.5em 0;
  padding: 0;
}
#user-infobox h2 a {
  color: inherit;
}
#user-infobox .logged-in {
  margin-top: 2em;
}
#user-infobox .logged-in,
#user-infobox .joined {
  font-size: 0.8em;
}
#user-infobox #infobox-text h2 {
  margin-top: 0;
  padding-top: 0;
}
#submitter {
  position: relative;
}
/****************************************************************************
 * New Review Request
 ****************************************************************************/
.new-review-request {
  background-color: #fefadf;
  background-image: url("../images/review_request_box_top_bg.ab6f3b1072c9.png");
}
.new-review-request label {
  color: #575012;
  font-weight: bold;
}
.new-review-request tr {
  padding-top: 4px;
}
.new-review-request .box-inner {
  background-image: url("../images/review_request_box_bottom_bg.e6485313ac61.png");
}
.new-review-request table,
#id_basedir,
#id_diff_path,
#id_parent_diff_path {
  width: 100%;
}
/****************************************************************************
 * clearfix hacks
 ****************************************************************************/
/*
 * clearfix hack. See http://www.webtoolkit.info/css-clearfix.html
 */
.clearfix {
  display: inline-block;
}
.clearfix:after {
  content: ".";
  display: block;
  clear: both;
  visibility: hidden;
  line-height: 0;
  height: 0;
}
html[xmlns] .clearfix {
  display: block;
}
* html .clearfix {
  height: 1%;
}

/* RB Colors */
/*
 * At least on Chrome, using points instead of pixels does not reliably
 * result in identical sizings when there are elements positioned inside
 * a <pre>. Pixels fixes this.
 */
/* Diff styles */
/* !(*%!(&^ (see http://www.positioniseverything.net/easyclearing.html) */
.clearfix {
  display: inline-block;
}
.clearfix:after {
  clear: both;
  content: ".";
  display: block;
  height: 0;
  visibility: hidden;
}
/* Hides from IE-mac \*/
* html .clearfix {
  height: 1%;
}
.clearfix {
  display: block;
}
/* End hide from IE-mac */
/****************************************************************************
 * Dashboard
 ****************************************************************************/
#dashboard_page {
  overflow: hidden;
  padding-bottom: 3px;
}
#dashboard_page #dashboard_container {
  display: none;
}
#dashboard_container {
  -moz-box-sizing: border-box;
  -webkit-box-sizing: border-box;
  box-sizing: border-box;
}
#dashboard_container .box-container,
#dashboard_container .box,
#dashboard_container .box-inner,
#dashboard_container #dashboard-wrapper {
  margin-bottom: 0;
  height: 100%;
}
#dashboard-main {
  background: white;
  border-left: 1px #888a85 solid;
  height: 100%;
  vertical-align: top;
  margin-left: 18em;
  -moz-border-radius: 0 5px 5px 0;
  -webkit-border-radius: 0 5px 5px 0;
  border-radius: 0 5px 5px 0;
}
#dashboard-main table:after {
  clear: both;
}
#dashboard-main .datagrid td.bugs a {
  text-decoration: underline;
  color: #0000c0;
}
#dashboard-main .datagrid-body-container {
  overflow-y: auto;
  margin-right: 1px;
}
#dashboard-main .datagrid-titlebox {
  -moz-border-radius: 0 5px 0 0;
  -webkit-border-radius: 0 5px 0 0;
  border-radius: 0 5px 0 0;
}
#dashboard-main .datagrid-title {
  background: transparent;
}
#dashboard-main .datagrid-wrapper {
  height: 100%;
}
#dashboard-main .main {
  padding: 0;
}
#dashboard-main .repository-column {
  white-space: nowrap;
}
#dashboard-main .diff-size-column {
  font-size: 90%;
}
#dashboard-main .diff-size-column.delete {
  color: darkred;
}
#dashboard-main .diff-size-column.insert {
  color: darkgreen;
}
#dashboard_actions {
  background: #333333;
  color: white;
  position: absolute;
  top: 0;
  bottom: 0;
  padding: 5.5em 0 0 0;
  width: 100%;
  -moz-border-radius: 5px 0 0 5px;
  -webkit-border-radius: 5px 0 0 5px;
  border-radius: 5px 0 0 5px;
  -moz-box-sizing: border-box;
  -webkit-box-sizing: border-box;
  box-sizing: border-box;
  opacity: 0.98;
  filter: alpha(opacity=98);
  -moz-opacity: 0.98;
}
#dashboard_actions a {
  color: white;
  font-size: 110%;
  text-decoration: none;
}
#dashboard_actions h2 {
  font-size: 130%;
}
#dashboard_actions ul {
  list-style: none;
  margin: 3em 1em 1em 1em;
  padding: 0;
  text-align: right;
}
#dashboard_actions ul li {
  margin: 1em 0;
  padding: 0;
}
#dashboard_actions .count {
  margin: 0 0 2em 0;
  text-align: center;
}
#dashboard_actions_content {
  padding: 8px;
  position: fixed;
  width: 18em;
  -moz-box-sizing: border-box;
  -webkit-box-sizing: border-box;
  box-sizing: border-box;
}
#dashboard_sidebar {
  float: left;
  top: 0;
  bottom: 1px;
  position: left;
  width: 18em;
}
#dashboard_sidebar.scrollable {
  float: none;
  overflow-y: auto;
  position: absolute;
}
#dashboard-navbar {
  list-style: none;
  margin: 0;
  padding: 0;
}
#dashboard-navbar li {
  display: block;
  list-style: none;
  margin: 0;
  text-align: left;
  word-break: break-all;
}
#dashboard-navbar li.has-url {
  cursor: pointer;
}
#dashboard-navbar li.has-url:hover .datagrid-sidebar-row,
#dashboard-navbar li.active .datagrid-sidebar-row {
  background: #dcdcdc;
}
#dashboard-navbar li.section {
  margin: 0 0 1em 0;
}
#dashboard-navbar li.section > .datagrid-sidebar-row .label {
  font-size: 110%;
  font-weight: bold;
}
#dashboard-navbar li a {
  color: black;
  text-decoration: none;
  padding: 0;
}
#dashboard-navbar li ul {
  list-style: none;
  margin: 0;
  padding: 0;
}
#dashboard-navbar li ul li.has-count .label {
  /* Give room for the counts. */
  margin-right: 4em;
}
#dashboard-navbar li ul li.new-subsection {
  margin-top: 1em;
}
#dashboard-navbar li ul li .label {
  display: inline-block;
  /* Give room for the icons. */
  margin-left: 20px;
}
#dashboard-navbar li ul li .rb-icon {
  position: absolute;
  top: 0.4em;
  left: 0.8em;
}
#dashboard-navbar li .count {
  position: absolute;
  top: 0.4em;
  right: 0.8em;
  background: #ffffff;
  border: 1px #d0d0d0 solid;
  font-size: 90%;
  padding: 0px 8px;
  margin-top: 0px;
  -moz-border-radius: 12px;
  -webkit-border-radius: 12px;
  border-radius: 12px;
}
#dashboard-navbar li .count.count-zero {
  color: #666666;
  background: #f1f1f1;
  border-color: #dcdcdc;
}
#dashboard-navbar li .datagrid-sidebar-row {
  line-height: 16px;
  padding: 0.4em 0.8em;
  position: relative;
}
#dashboard-navbar li .datagrid-sidebar-row .rb-icon {
  vertical-align: middle;
}
#dashboard-wrapper {
  background-color: #e9e9e9;
  position: relative;
  -moz-border-radius: 5px;
  -webkit-border-radius: 5px;
  border-radius: 5px;
}
#dashboard-navbar #user_page_profile {
  margin-bottom: 1em;
}
#dashboard-navbar #user_page_profile .nickname {
  font-size: 1.4em;
  margin-bottom: 0;
  padding-bottom: 0;
}
#dashboard-navbar #user_page_profile .email,
#dashboard-navbar #user_page_profile .group {
  color: #0000C0;
}
#dashboard-navbar #user_page_profile .fn,
#dashboard-navbar #user_page_profile .email {
  font-size: 1.2em;
}
#dashboard-navbar #user_page_profile .logged-in {
  margin-top: 1.5em;
}

.rb-icon {
  background-repeat: no-repeat;
  display: inline-block;
  overflow: hidden;
  text-indent: -99999px;
  vertical-align: middle;
  background-image: url("../images/icons.2eed2f830119.png");
  background-image: url("../images/icons.2eed2f830119.png");
  zoom: 1;
  *display: inline;
}
@media (-webkit-min-device-pixel-ratio: 1.5), (min--moz-device-pixel-ratio: 1.5), (-o-min-device-pixel-ratio: 3/2), (min-resolution: 1.5dppx) {
  .rb-icon {
    background-image: url("../images/icons@2x.76271275352d.png");
    background-size: 155px 109px;
  }
}
/*
 * NOTE: Keep these in the same order as found in the spritesheet.
 */
.rb-icon-issue-dropped {
  background-position: -5px -5px;
  width: 16px;
  height: 16px;
}
.rb-icon-issue-open {
  background-position: -26px -5px;
  width: 16px;
  height: 16px;
}
.rb-icon-issue-resolved {
  background-position: -47px -5px;
  width: 16px;
  height: 16px;
}
.rb-icon-collapse-review {
  background-position: -68px -5px;
  width: 18px;
  height: 18px;
}
.rb-icon-expand-review {
  background-position: -91px -5px;
  width: 18px;
  height: 18px;
}
.rb-icon-collapse {
  background-position: -114px -5px;
  width: 14px;
  height: 14px;
}
.rb-icon-expand {
  background-position: -5px -28px;
  width: 14px;
  height: 14px;
}
.rb-icon-datagrid-comment-draft {
  background-position: -24px -28px;
  width: 16px;
  height: 16px;
}
.rb-icon-datagrid-comment-shipit {
  background-position: -45px -28px;
  width: 16px;
  height: 16px;
}
.rb-icon-datagrid-comment {
  background-position: -66px -28px;
  width: 16px;
  height: 16px;
}
.rb-icon-shipit-checkmark {
  background-position: -87px -28px;
  width: 9px;
  height: 10px;
}
.rb-icon-datagrid-new-updates {
  background-position: -101px -28px;
  width: 18px;
  height: 16px;
}
.rb-icon-edit {
  background-position: -124px -28px;
  width: 20px;
  height: 16px;
}
.rb-icon-delete {
  background-position: -5px -49px;
  width: 12px;
  height: 12px;
}
.rb-icon-diff-collapse-chunk {
  background-position: -22px -49px;
  width: 14px;
  height: 14px;
}
.rb-icon-diff-expand-above {
  background-position: -41px -49px;
  width: 28px;
  height: 14px;
}
.rb-icon-diff-expand-all {
  background-position: -74px -49px;
  width: 14px;
  height: 14px;
}
.rb-icon-diff-expand-below {
  background-position: -93px -49px;
  width: 28px;
  height: 14px;
}
.rb-icon-diff-expand-header {
  background-position: -126px -49px;
  width: 14px;
  height: 14px;
}
.rb-icon-shipit {
  background-position: -5px -68px;
  width: 16px;
  height: 16px;
}
.rb-icon-admin-enabled {
  background-position: -5px -68px;
  width: 16px;
  height: 16px;
}
.rb-icon-admin-disabled {
  background-position: -26px -68px;
  width: 16px;
  height: 16px;
}
.rb-icon-admin-add {
  background-position: -47px -68px;
  width: 16px;
  height: 16px;
}
.rb-icon-star-off {
  background-position: -68px -68px;
  width: 16px;
  height: 16px;
}
.rb-icon-star-on {
  background-position: -89px -68px;
  width: 16px;
  height: 16px;
}
.rb-icon-warning {
  background-position: -110px -68px;
  width: 16px;
  height: 16px;
}
.rb-icon-search {
  background-position: -129px -66px;
  width: 20px;
  height: 20px;
}
.rb-icon-search-dark {
  background-position: -3px -86px;
  width: 20px;
  height: 20px;
}
.rb-icon-admin-collapse {
  background-position: -25px -89px;
  width: 20px;
  height: 13px;
}
.rb-icon-admin-expand {
  background-position: -50px -89px;
  width: 20px;
  height: 13px;
}
.rb-icon-jump-to {
  background-position: -76px -86px;
  width: 21px;
  height: 18px;
}
.rb-icon-range-slider {
  background-position: -100px -87px;
  width: 9px;
  height: 18px;
}
.rb-icon-download {
  background-position: -114px -90px;
  width: 14px;
  height: 14px;
}

#search_results {
  margin: 3em;
}
#search_results .paginator {
  border-top: 1px #E0E0E0 solid;
  font-size: 120%;
  margin-top: 2em;
  padding-top: 2em;
}
#search_results .paginator a {
  color: blue;
  text-decoration: none;
}
#search_results .paginator a,
#search_results .paginator b {
  margin-right: 0.5em;
}
#search_results .search-results-filters {
  border-bottom: 1px #cccccc solid;
  margin-bottom: 2em;
}
#search_results .search-results-filters a {
  color: #666;
  display: inline-block;
  font-size: 130%;
  text-decoration: none;
  padding: 0.5em 1em;
  border: 1px transparent solid;
  border-top-width: 2px;
}
#search_results .search-results-filters a.active {
  color: black;
  border: 1px #cccccc solid;
  border-bottom: 1px white solid;
  border-top-width: 2px;
}
#search_results .search-results-stats {
  color: #444444;
  font-size: 110%;
  padding-bottom: 2em;
  margin-left: 1em;
}
#search_results .search-result {
  display: inline-block;
  display: block;
  line-height: 1.5;
  margin-bottom: 2em;
}
#search_results .search-result:after {
  clear: both;
  content: ".";
  display: block;
  height: 0;
  visibility: hidden;
}
#search_results .search-result a {
  text-decoration: none;
}
#search_results .search-result h2 {
  font-size: 140%;
  font-weight: normal;
  margin: 0;
}
#search_results .search-result .search-result-by {
  color: #555555;
  font-size: 85%;
  margin-top: 0.3em;
}
#search_results .search-result .search-result-body {
  float: left;
}
#search_results .search-result .search-result-excerpt {
  margin-top: 0.3em;
  width: 60em;
}
#search_results .search-result .search-result-icon {
  float: left;
  margin-right: 1em;
}
#search_results .search-result .search-result-links {
  margin-top: 1em;
}
#search_results .search-result .search-result-username {
  font-size: 120%;
}
#search_results .search-result .search-result-user-groups {
  margin-top: 1em;
}
