{% load i18n %}
{% if request.user.is_authenticated %}
 <li class="primary"><a id="review-link" href="#">{% trans "Review" %}</a></li>
 <li class="primary"><a id="shipit-link" href="#">{% trans "Ship It!" %}</a></li>
{% endif %}
