#page-container {
  display: none;
}
#HTMLReporter {
  font-size: 11px;
  font-family: Monaco, "Lucida Console", monospace;
  line-height: 14px;
  color: #333333;
  margin: 1em;
}
#HTMLReporter.showDetails #details {
  display: block;
}
#HTMLReporter.showDetails .detailsMenuItem {
  font-weight: bold;
  text-decoration: underline;
}
#HTMLReporter.showDetails .summaryMenuItem {
  font-weight: normal;
  text-decoration: inherit;
}
#HTMLReporter.showDetails .summaryMenuItem:hover {
  text-decoration: underline;
}
#HTMLReporter.showDetails .summary {
  display: none;
}
#HTMLReporter a {
  text-decoration: none;
}
#HTMLReporter a:hover {
  text-decoration: underline;
}
#HTMLReporter p,
#HTMLReporter h1,
#HTMLReporter h2,
#HTMLReporter h3,
#HTMLReporter h4,
#HTMLReporter h5,
#HTMLReporter h6 {
  margin: 0;
  line-height: 14px;
}
#HTMLReporter #details {
  display: none;
}
#HTMLReporter #details .specDetail {
  margin-bottom: 28px;
}
#HTMLReporter #details .specDetail .description {
  display: block;
  color: white;
  background-color: #b03911;
}
#HTMLReporter #jasmine_content {
  position: fixed;
  right: 100%;
}
#HTMLReporter .banner,
#HTMLReporter .symbolSummary,
#HTMLReporter .summary,
#HTMLReporter .resultMessage,
#HTMLReporter .specDetail .description,
#HTMLReporter .alert .bar,
#HTMLReporter .stackTrace {
  padding-left: 9px;
  padding-right: 9px;
}
#HTMLReporter .version {
  color: #666666;
}
#HTMLReporter .banner {
  margin-top: 14px;
}
#HTMLReporter .duration {
  color: #666666;
  float: right;
}
#HTMLReporter .symbolSummary {
  overflow: hidden;
  *zoom: 1;
  margin: 14px 0;
}
#HTMLReporter .symbolSummary li {
  display: block;
  float: left;
  height: 7px;
  width: 14px;
  margin-bottom: 7px;
  font-size: 16px;
}
#HTMLReporter .symbolSummary li.passed {
  font-size: 14px;
}
#HTMLReporter .symbolSummary li.passed:before {
  color: #5e7d00;
  content: "\02022";
}
#HTMLReporter .symbolSummary li.failed {
  line-height: 9px;
}
#HTMLReporter .symbolSummary li.failed:before {
  color: #b03911;
  content: "x";
  font-weight: bold;
  margin-left: -1px;
}
#HTMLReporter .symbolSummary li.skipped {
  font-size: 14px;
}
#HTMLReporter .symbolSummary li.skipped:before {
  color: #bababa;
  content: "\02022";
}
#HTMLReporter .symbolSummary li.pending {
  line-height: 11px;
}
#HTMLReporter .symbolSummary li.pending:before {
  color: #aaaaaa;
  content: "-";
}
#HTMLReporter .exceptions {
  color: #fff;
  float: right;
  margin-top: 5px;
  margin-right: 5px;
}
#HTMLReporter .bar {
  line-height: 28px;
  font-size: 14px;
  display: block;
  color: #eee;
}
#HTMLReporter .runningAlert {
  background-color: #666666;
}
#HTMLReporter .skippedAlert {
  background-color: #aaaaaa;
}
#HTMLReporter .skippedAlert:first-child {
  background-color: #333333;
}
#HTMLReporter .skippedAlert:hover {
  text-decoration: none;
  color: white;
  text-decoration: underline;
}
#HTMLReporter .passingAlert {
  background-color: #a6b779;
}
#HTMLReporter .passingAlert:first-child {
  background-color: #5e7d00;
}
#HTMLReporter .failingAlert {
  background-color: #cf867e;
}
#HTMLReporter .failingAlert:first-child {
  background-color: #b03911;
}
#HTMLReporter .results {
  margin-top: 14px;
}
#HTMLReporter .resultsMenu,
#HTMLReporter .resultsMenu a {
  background-color: #fff;
  color: #333333;
}
#HTMLReporter .summaryMenuItem {
  font-weight: bold;
  text-decoration: underline;
}
#HTMLReporter .summary {
  margin-top: 14px;
}
#HTMLReporter .summary .suite .suite,
#HTMLReporter .summary .specSummary {
  margin-left: 14px;
}
#HTMLReporter .summary .specSummary.passed a {
  color: #5e7d00;
}
#HTMLReporter .summary .specSummary.failed a {
  color: #b03911;
}
#HTMLReporter .description + .suite {
  margin-top: 0;
}
#HTMLReporter .suite {
  margin-top: 14px;
}
#HTMLReporter .suite a {
  color: #333333;
}
#HTMLReporter .resultMessage {
  padding-top: 14px;
  color: #333333;
}
#HTMLReporter .resultMessage span.result {
  display: block;
}
#HTMLReporter .stackTrace {
  background: white;
  border: 1px solid #ddd;
  color: #666666;
  line-height: 18px;
  margin: 5px 0 0 0;
  max-height: 224px;
  overflow: auto;
  white-space: pre;
}
