{% extends "base.html" %}
{% load djblets_deco i18n %}

{% block title %}{% trans "Permission Denied" %}{% endblock %}

{% block content %}
<div class="permissions-offset">
 {%  box "important" %}
 <div class="box-head">
  <h1 class="box-title">{% trans "You don't have access to this review request." %}</h1>
 </div>
 <div class="box-main">
  <p>
  {% blocktrans %}
   This review request is private. You must be a requested reviewer, either
   directly or on a requested group, and have permission to access the
   repository in order to view this review request.
  {% endblocktrans %}
   </p>
  </div>
 {%  endbox %}
</div>
{% endblock %}
