/* RB Colors */
/*
 * At least on Chrome, using points instead of pixels does not reliably
 * result in identical sizings when there are elements positioned inside
 * a <pre>. Pixels fixes this.
 */
/* Diff styles */
/* !(*%!(&^ (see http://www.positioniseverything.net/easyclearing.html) */
.clearfix {
  display: inline-block;
}
.clearfix:after {
  clear: both;
  content: ".";
  display: block;
  height: 0;
  visibility: hidden;
}
/* Hides from IE-mac \*/
* html .clearfix {
  height: 1%;
}
.clearfix {
  display: block;
}
/* End hide from IE-mac */
.dnd-overlay {
  opacity: 0.8;
  filter: alpha(opacity=80);
  -moz-opacity: 0.8;
  position: fixed;
  left: 0;
  top: 0;
  width: 100%;
  height: 100%;
  background: black;
  color: #DDDDDD;
  font-size: 200%;
  font-weight: bold;
  text-align: center;
  vertical-align: middle;
  z-index: 130;
}
