.c, .com, /* Comment */
.cm, /* Comment.Multiline */
.c1 /* Comment.Single */
{ color: #0000C0; font-style: italic }
.err { border: 1px solid #FF0000 } /* Error */
.k { font-weight: bold } /* Keyword */
.o { color: black; } /* Operator */
.cp { color: #612B00 } /* Comment.Preproc */
.gd { color: #A00000 } /* Generic.Deleted */
.ge { font-style: italic } /* Generic.Emph */
.gr { color: #FF0000 } /* Generic.Error */
.gh { color: #000080; font-weight: bold } /* Generic.Heading */
.gi { color: #004000 } /* Generic.Inserted */
.go { color: #808080 } /* Generic.Output */
.gp { color: #000080; font-weight: bold } /* Generic.Prompt */
.gs { font-weight: bold } /* Generic.Strong */
.gu { color: #800080; font-weight: bold } /* Generic.Subheading */
.gt { color: #0040D0 } /* Generic.Traceback */
.kc { font-weight: bold } /* Keyword.Constant */
.kd { font-weight: bold } /* Keyword.Declaration */
.kp { color: #AA22FF } /* Keyword.Pseudo */
.kr, .kwd { font-weight: bold } /* Keyword.Reserved */
.kt, { font-weight: bold } /* Keyword.Type */
.m, .lit { color: darkred; font-weight: bold; } /* Literal.Number */
.s, .str, .atv { color: darkred; } /* Literal.String */
.na, .atn { color: #000080 } /* Name.Attribute */
.nb { font-weight: bold; } /* Name.Builtin */
.nc { color: black } /* Name.Class */
.no { color: #880000 } /* Name.Constant */
.nd { color: darkBlue; font-weight: bold} /* Name.Decorator */
.ni { color: #999999; font-weight: bold } /* Name.Entity */
.ne { color: #D2413A; font-weight: bold } /* Name.Exception */
.nf, .fun { color: black } /* Name.Function */
.nl { color: black } /* Name.Label */
.nn { color: black } /* Name.Namespace */
.nt, .tag, .tag { color: #000080; font-weight: bold } /* Name.Tag */
.nv, .dec, .var { color: black } /* Name.Variable */
.ow { font-weight: bold } /* Operator.Word */
.mf { color: darkred; font-weight: bold } /* Literal.Number.Float */
.mh { color: darkred; font-weight: bold } /* Literal.Number.Hex */
.mi { color: darkred; font-weight: bold } /* Literal.Number.Integer */
.mo { color: darkred; font-weight: bold } /* Literal.Number.Oct */
.sb { color: #BB4444 } /* Literal.String.Backtick */
.sc { color: #BB4444 } /* Literal.String.Char */
.sd { color: #0000C0; } /* Literal.String.Doc */
.s2 { color: #BB4444 } /* Literal.String.Double */
.se { color: #BB6622; font-weight: bold } /* Literal.String.Escape */
.sh { color: #BB4444 } /* Literal.String.Heredoc */
.si { color: red; font-weight: bold } /* Literal.String.Interpol */
.sx { color: #008000 } /* Literal.String.Other */
.sr { color: #BB6688 } /* Literal.String.Regex */
.s1 { color: #BB4444 } /* Literal.String.Single */
.ss { color: #B8860B } /* Literal.String.Symbol */
.bp { color: darkred; font-weight: bold; } /* Name.Builtin.Pseudo */
.vc { color: #B8860B } /* Name.Variable.Class */
.vg { color: #B8860B } /* Name.Variable.Global */
.vi { color: #B8860B } /* Name.Variable.Instance */
.il { color: #666666 } /* Literal.Number.Integer.Long */
.cs { color: red; font-weight: bold; font-style: italic; }
