{% extends "admin/base_site.html" %}
{% load i18n log %}

{% block coltype %}colMS{% endblock %}
{% block bodyclass %}dashboard{% endblock %}
{% block breadcrumbs %}{% endblock %}

{% block content %}
<div id="content-main">

{% if app_list %}
{%  for app in app_list %}
 <fieldset class="module">
  <h2>{% blocktrans with app.name as name %}{{name}}{% endblocktrans %}</h2>

  <table summary="{% blocktrans with app.name as name %}Models available in the {{name}} application.{% endblocktrans %}">
{%   for model in app.models %}
    <tr>
{%    if model.perms.change %}
     <th scope="row"><a href="{{model.admin_url}}">{{model.name}}</a></th>
{%    else %}
     <th scope="row">{{model.name}}</th>
{%    endif %}

{%    if model.perms.add %}
     <td><a href="{{model.admin_url}}add/" class="addlink">{% trans "Add" %}</a></td>
{%    else %}
     <td>&nbsp;</td>
{%    endif %}

{%    if model.perms.change %}
     <td><a href="{{model.admin_url}}" class="changelink">{% trans "Change" %}</a></td>
{%    else %}
     <td>&nbsp;</td>
{%    endif %}
    </tr>
{%   endfor %}
   </table>
  </fieldset>
{%  endfor %}
{% else %}
 <p>{% trans "You don't have permission to edit anything." %}</p>
{% endif %}
</div>
{% endblock %}

{% block sidebar %}
<div id="content-related">
 <div class="module" id="recent-actions-module">
  <h2>{% trans "Recent Actions" %}</h2>
  <h3>{% trans "My Actions" %}</h3>
{% get_admin_log 10 as admin_log for_user user %}
{% if not admin_log %}
  <p>{% trans "None available" %}</p>
{% else %}
  <ul class="actionlist">
{%  for entry in admin_log %}
   <li class="{% if entry.is_addition %}addlink{% endif %}{% if entry.is_change %}changelink{% endif %}{% if entry.is_deletion %}deletelink{% endif %}">{% if not entry.is_deletion %}<a href="{{ entry.get_admin_url }}">{% endif %}{{ entry.object_repr|escape }}{% if not entry.is_deletion %}</a>{% endif %}<br /><span class="mini quiet">{% filter capfirst %}{% trans entry.content_type.name %}{% endfilter %}</span></li>
{%  endfor %}
  </ul>
{% endif %}
 </div>
</div>
{% endblock %}

