{% extends "admin/change_list.html" %}

{% block scripts %}
{{block.super}}

<script>
function showInstructionsBox(url, title) {
    var $el = $('<div>')
        .load(url, function() {
            $el.modalBox({
                title: title
            });
        });
}

$(document)
    .on('click', '.action-hooks-setup', function(evt) {
        evt.preventDefault();
        showInstructionsBox(evt.target.href,
                            gettext('Hooks Setup Instructions'));
    })
    .on('click', '.action-rbtools-setup', function(evt) {
        evt.preventDefault();
        showInstructionsBox(evt.target.href,
                            gettext('RBTools Setup Instructions'));
    });
</script>
{% endblock %}
