/* RB Colors */
/*
 * At least on Chrome, using points instead of pixels does not reliably
 * result in identical sizings when there are elements positioned inside
 * a <pre>. Pixels fixes this.
 */
/* Diff styles */
/* !(*%!(&^ (see http://www.positioniseverything.net/easyclearing.html) */
.clearfix {
  display: inline-block;
}
.clearfix:after {
  clear: both;
  content: ".";
  display: block;
  height: 0;
  visibility: hidden;
}
/* Hides from IE-mac \*/
* html .clearfix {
  height: 1%;
}
.clearfix {
  display: block;
}
/* End hide from IE-mac */
body {
  background-color: white;
  color: black;
  font-family: Verdana, Arial, Helvetica, Sans-Serif;
  font-size: 8pt;
  margin: 0;
  padding: 0;
}
form th {
  text-align: right;
  vertical-align: top;
}
h1 {
  font-size: 10pt;
}
h3 {
  margin-top: 2em;
}
img {
  border: 0;
}
label.required:after {
  content: ' *';
  font-weight: bold;
  font-size: 10px;
  color: darkred;
}
pre,
textarea,
.CodeMirror,
.editable.field-text-area {
  font-size: 13px;
  font-family: monospace;
  line-height: 15px;
}
span.label-draft {
  color: #008000;
}
span.label-submitted {
  color: #b7a93e;
}
span.label-discarded {
  color: #fe5757;
}
td.help {
  font-size: 10px;
  color: #333;
  padding-top: 0;
}
textarea {
  resize: none;
}
ul.errorlist {
  color: #DD0000;
  display: inline;
  font-weight: bold;
  margin: 0 0 0 15px;
  padding: 4px 8px;
}
ul.errorlist li {
  display: inline;
}
.loading img {
  margin-right: 4px;
  vertical-align: text-bottom;
}
/****************************************************************************
 * Buttons
 ****************************************************************************/
.btn,
input[type="button"],
input[type="submit"] {
  cursor: pointer;
  display: inline-block;
  background-color: #e6e6e6;
  background-repeat: no-repeat;
  background-image: -webkit-gradient(linear, 0 0, 0 100%, from(#ffffff), color-stop(25%, #ffffff), to(#e6e6e6));
  background-image: -webkit-linear-gradient(#ffffff, #ffffff 25%, #e6e6e6);
  background-image: -moz-linear-gradient(top, #ffffff, #ffffff 25%, #e6e6e6);
  background-image: -ms-linear-gradient(#ffffff, #ffffff 25%, #e6e6e6);
  background-image: -o-linear-gradient(#ffffff, #ffffff 25%, #e6e6e6);
  background-image: linear-gradient(#ffffff, #ffffff 25%, #e6e6e6);
  filter: progid:DXImageTransform.Microsoft.gradient(startColorstr='#ffffff', endColorstr='#e6e6e6', GradientType=0);
  padding: 3px 8px;
  text-shadow: 0 1px 1px rgba(255, 255, 255, 0.75);
  color: #333;
  font-size: 13px;
  line-height: normal;
  border: 1px solid #ccc;
  border-bottom-color: #bbb;
  -moz-border-radius: 4px;
  -webkit-border-radius: 4px;
  border-radius: 4px;
  -moz-box-shadow: inset 0 1px 0 rgba(255, 255, 255, 0.2) 0 1px 2px rgba(0, 0, 0, 0.05);
  -webkit-box-shadow: inset 0 1px 0 rgba(255, 255, 255, 0.2) 0 1px 2px rgba(0, 0, 0, 0.05);
  box-shadow: inset 0 1px 0 rgba(255, 255, 255, 0.2) 0 1px 2px rgba(0, 0, 0, 0.05);
  -webkit-transition: 0.1s linear all;
  -moz-transition: 0.1s linear all;
  transition: 0.1s linear all;
}
.btn:hover,
input[type="button"]:hover,
input[type="submit"]:hover {
  background-position: 0 -15px;
  color: #333;
  text-decoration: none;
}
.btn.primary,
input[type="button"].primary,
input[type="submit"].primary,
.btn.default,
input[type="button"].default,
input[type="submit"].default {
  color: white;
  text-shadow: none;
  background-color: #44679a;
  background-repeat: repeat-x;
  background-image: -khtml-gradient(linear, left top, left bottom, from(#7e9bc6), to(#44679a));
  background-image: -moz-linear-gradient(top, #7e9bc6, #44679a);
  background-image: -ms-linear-gradient(top, #7e9bc6, #44679a);
  background-image: -webkit-gradient(linear, left top, left bottom, color-stop(0%, #7e9bc6), color-stop(100%, #44679a));
  background-image: -webkit-linear-gradient(top, #7e9bc6, #44679a);
  background-image: -o-linear-gradient(top, #7e9bc6, #44679a);
  background-image: linear-gradient(top, #7e9bc6, #44679a);
  filter: progid:DXImageTransform.Microsoft.gradient(startColorstr='#7e9bc6', endColorstr='#44679a', GradientType=0);
  text-shadow: 0 -1px 0 rgba(0, 0, 0, 0.25);
  border-color: #44679a #44679a #2d4365;
  border-color: rgba(0, 0, 0, 0.1) rgba(0, 0, 0, 0.1) rgba(0, 0, 0, 0.25);
}
.btn:active,
input[type="button"]:active,
input[type="submit"]:active {
  -moz-box-shadow: inset 0 2px 4px rgba(0, 0, 0, 0.25) 0 1px 2px rgba(0, 0, 0, 0.05);
  -webkit-box-shadow: inset 0 2px 4px rgba(0, 0, 0, 0.25) 0 1px 2px rgba(0, 0, 0, 0.05);
  box-shadow: inset 0 2px 4px rgba(0, 0, 0, 0.25) 0 1px 2px rgba(0, 0, 0, 0.05);
}
.btn.disabled,
input[type="button"].disabled,
input[type="submit"].disabled {
  cursor: default;
  background-image: none;
  filter: progid:DXImageTransform.Microsoft.gradient(enabled = false);
  opacity: 0.65;
  filter: alpha(opacity=65);
  -moz-opacity: 0.65;
  -moz-box-shadow: none;
  -webkit-box-shadow: none;
  box-shadow: none;
}
.btn[disabled],
input[type="button"][disabled],
input[type="submit"][disabled] {
  cursor: default;
  background-image: none;
  filter: progid:DXImageTransform.Microsoft.gradient(enabled = false);
  opacity: 0.65;
  filter: alpha(opacity=65);
  -moz-opacity: 0.65;
  -moz-box-shadow: none;
  -webkit-box-shadow: none;
  box-shadow: none;
}
.btn.large,
input[type="button"].large,
input[type="submit"].large {
  font-size: 130%;
  line-height: normal;
  padding: 9px 14px 9px;
  -moz-border-radius: 6px;
  -webkit-border-radius: 6px;
  border-radius: 6px;
}
.btn.small,
input[type="button"].small,
input[type="submit"].small {
  padding: 7px 9px 7px;
  font-size: 11px;
}
.btn.danger,
input[type="button"].danger,
input[type="submit"].danger {
  color: white !important;
  background-color: #c43c35;
  background-repeat: repeat-x;
  background-image: -khtml-gradient(linear, left top, left bottom, from(#ee5f5b), to(#c43c35));
  background-image: -moz-linear-gradient(top, #ee5f5b, #c43c35);
  background-image: -ms-linear-gradient(top, #ee5f5b, #c43c35);
  background-image: -webkit-gradient(linear, left top, left bottom, color-stop(0%, #ee5f5b), color-stop(100%, #c43c35));
  background-image: -webkit-linear-gradient(top, #ee5f5b, #c43c35);
  background-image: -o-linear-gradient(top, #ee5f5b, #c43c35);
  background-image: linear-gradient(top, #ee5f5b, #c43c35);
  filter: progid:DXImageTransform.Microsoft.gradient(startColorstr='#ee5f5b', endColorstr='#c43c35', GradientType=0);
  text-shadow: 0 -1px 0 rgba(0, 0, 0, 0.25);
  border-color: #c43c35 #c43c35 #882a25;
  border-color: rgba(0, 0, 0, 0.1) rgba(0, 0, 0, 0.1) rgba(0, 0, 0, 0.25);
}
button.btn::-moz-focus-inner,
input[type=submit].btn::-moz-focus-inner {
  padding: 0;
  border: 0;
}
/****************************************************************************
 * Banners
 ****************************************************************************/
.banner {
  background: #cdff9c;
  border: 1px #4d9c00 solid;
  margin-bottom: 10px;
  padding: 8px 10px;
  z-index: 90;
}
.banner > h1 {
  display: inline;
  margin-right: 0.2em;
  padding: 0;
}
.banner > input {
  margin-left: 10px;
}
/****************************************************************************
 * Box classes
 ****************************************************************************/
.box,
.modalbox {
  background-color: #ffffff;
  background-image: url('../images/box_top_bg.png');
  background-position: top left;
  background-repeat: repeat-x;
  border: 1px #888a85 solid;
  -moz-border-radius: 6px;
  -webkit-border-radius: 6px;
  border-radius: 6px;
  -moz-box-shadow: 1px 1px 2px rgba(0, 0, 0, 0.15);
  -webkit-box-shadow: 1px 1px 2px rgba(0, 0, 0, 0.15);
  box-shadow: 1px 1px 2px rgba(0, 0, 0, 0.15);
}
.box.important {
  background-color: #fdf2a0;
  background-image: url('../images/gold_box_top_bg.png');
  border: 1px #b7a93e solid;
  padding: 0 10px;
}
.box.important .box-inner {
  background: url("../images/important.png") no-repeat top left;
  padding: 0 0 0 50px;
}
.box.tip {
  background-color: #fdf2a0;
  background-image: url('../images/gold_box_top_bg.png');
  border: 1px #b7a93e solid;
  padding: 0 10px;
}
.box.tip .box-inner {
  background: transparent;
}
.box.tip h1 {
  padding-left: 0;
}
.box.yay {
  background-color: #fdf2a0;
  background-image: url('../images/gold_box_top_bg.png');
  border: 1px #b7a93e solid;
  padding: 0 10px;
}
.box.yay .box-inner {
  background: transparent;
  position: relative;
}
.box.yay h1 {
  padding-left: 2.5em;
}
.box.yay img {
  float: left;
  left: -5px;
  margin: 0;
  padding: 0;
  position: absolute;
  top: -20px;
}
.box .star {
  cursor: pointer;
}
.admin h1.title,
.box-head,
.modalbox-title {
  background: #c6dcf3;
  border-bottom: 1px #728eac solid;
  margin: 0;
  padding: 0.4em 0.6em;
  -moz-border-radius: 6px 6px 0 0;
  -webkit-border-radius: 6px 6px 0 0;
  border-radius: 6px 6px 0 0;
}
.box-title,
.modalbox-title {
  font-size: 120%;
}
.box-title {
  margin: 0;
  padding: 0;
}
.box-main {
  padding: 5px;
}
.box-container {
  margin-bottom: 10px;
  width: 100%;
}
.box-inner,
.modalbox-inner {
  background-image: url('../images/box_bottom_bg.png');
  background-position: bottom left;
  background-repeat: repeat-x;
  -moz-border-radius: 6px;
  -webkit-border-radius: 6px;
  border-radius: 6px;
  padding-bottom: 1px\9;
  /* IE wants this. The \9 targets IE 7,8,9. */
}
.box-inner form {
  margin: 0;
}
.modalbox,
.modalbox-bg {
  z-index: 100;
}
.datagrid-title,
.datagrid-titlebox {
  background: #c6dcf3 !important;
  -moz-border-radius: 6px 6px 0 0;
  -webkit-border-radius: 6px 6px 0 0;
  border-radius: 6px 6px 0 0;
}
.datagrid-main {
  -moz-border-radius: 6px;
  -webkit-border-radius: 6px;
  border-radius: 6px;
}
.datagrid-header {
  border-top: 0 !important;
}
.datagrid-header:last-child {
  border-right: 0;
}
.datagrid-menu {
  border: 1px #888A85 solid;
}
.datagrid tr:last-child td:first-child {
  -moz-border-radius: 0 0 0 6px;
  -webkit-border-radius: 0 0 0 6px;
  border-radius: 0 0 0 6px;
}
.datagrid tr:last-child td:last-child {
  -moz-border-radius: 0 0 6px 0;
  -webkit-border-radius: 0 0 6px 0;
  border-radius: 0 0 6px 0;
}
.datagrid td:last-child {
  border-right: 0;
}
.datagrid .issue-count,
.datagrid .shipit-count {
  display: inline-block;
  font-weight: bold;
  padding: 0.05em 0.8em;
  white-space: nowrap;
  -moz-border-radius: 10px;
  -webkit-border-radius: 10px;
  border-radius: 10px;
}
.datagrid .issue-count {
  border: 1px #9c8b00 solid;
  color: #205003;
  background-color: #ffe303;
  background-repeat: no-repeat;
  background-image: -webkit-gradient(linear, 0 0, 0 100%, from(#fff49c), color-stop(25%, #fff49c), to(#ffe303));
  background-image: -webkit-linear-gradient(#fff49c, #fff49c 25%, #ffe303);
  background-image: -moz-linear-gradient(top, #fff49c, #fff49c 25%, #ffe303);
  background-image: -ms-linear-gradient(#fff49c, #fff49c 25%, #ffe303);
  background-image: -o-linear-gradient(#fff49c, #fff49c 25%, #ffe303);
  background-image: linear-gradient(#fff49c, #fff49c 25%, #ffe303);
  filter: progid:DXImageTransform.Microsoft.gradient(startColorstr='#fff49c', endColorstr='#ffe303', GradientType=0);
}
.datagrid .issue-count .issue-icon {
  color: #695d00;
  display: inline-block;
  padding: 0 3px 0 1px;
  text-align: center;
}
.datagrid .shipit-count {
  background-image: url("../images/shipit_bg.png");
  background-position: top left;
  background-repeat: repeat-x;
  background-color: #6bc810;
  border: 1px #478a06 solid;
  color: #205003;
}
.server-error-box .response-data {
  margin-top: 2em;
}
/****************************************************************************
 * Modal Boxes
 ****************************************************************************/
.modalbox {
  margin: 10px;
}
.modalbox .modalbox-contents {
  margin: 10px;
  position: relative;
  /* Makes this the offsetParent for calculations. */
}
.modalbox .modalbox-buttons {
  position: absolute;
  margin: 10px;
  text-align: right;
  bottom: 0;
  right: 0;
}
.modalbox .modalbox-buttons input {
  margin-left: 10px;
}
/****************************************************************************
 * Inline editor forms
 ****************************************************************************/
.editicon {
  margin-left: 4px;
}
.editicon .rb-icon {
  vertical-align: bottom;
}
.editable,
.editicon {
  line-height: 14px;
}
.inline-editor-form textarea,
.text-editor textarea {
  border: 1px #b8b5a0 solid;
  margin: 0;
  outline: none;
  padding: 10px;
  -moz-box-sizing: border-box;
  -webkit-box-sizing: border-box;
  box-sizing: border-box;
  /*
     * This prevents extra spacing below a text area in different browsers.
     * See http://stackoverflow.com/questions/7144843/extra-space-under-textarea-differs-along-browsers
     */
  vertical-align: top;
}
.inline-editor-form {
  display: block;
  margin: 0;
  padding: 0;
  white-space: nowrap;
}
.inline-editor-form .buttons input[type='button'] {
  margin-left: 6px;
  margin-right: 0;
}
.inline-editor-form .buttons input[type='button']:first-child {
  margin-left: 0;
}
.inline-editor-form input[type="text"] {
  border: 1px #888a85 solid;
  padding: 1px 2px;
}
.inline-editor-form input[type] + .buttons {
  padding-left: 6px;
}
.inline-editor-form textarea {
  margin-top: 5px;
}
.inline-editor-form textarea + .buttons,
.inline-editor-form .text-editor + .buttons {
  margin-top: 6px;
  white-space: normal;
}
.inline-editor-form .enable-markdown {
  margin-left: 1em;
}
.inline-editor-form .enable-markdown label {
  color: black;
  font-size: 8pt !important;
  font-weight: normal;
}
.loading-indicator {
  display: inline;
}
.text-editor {
  margin: 5px 0 0 0;
}
.text-editor textarea,
.text-editor .CodeMirror {
  margin: 0;
}
/****************************************************************************
 * Forms
 ****************************************************************************/
form table td {
  padding: 8px 8px 4px 4px;
}
form table td.label {
  text-align: right;
  font-weight: bold;
}
.formdlg tr {
  padding-top: 4px;
}
.formdlg td.label {
  white-space: nowrap;
}
.formdlg .error {
  color: #DD0000;
  font-weight: bold;
  margin-bottom: 10px;
  padding: 4px 8px;
}
.formdlg .errorlist {
  display: block;
  list-style: none;
  margin: 0;
  padding: 0;
}
.formdlg .errorlist li {
  display: list-item;
  padding: 2px 4px;
}
.formdlg .spinner {
  float: left;
}
.formdlg .spinner img {
  vertical-align: top;
}
.formdlg .spinner h1 {
  display: inline;
  margin-left: 10px;
}
/****************************************************************************
 * Account page
 ****************************************************************************/
.prefsbuttons {
  padding: 1em;
  text-align: center;
}
.prefsgroup ul {
  list-style: none;
  padding-left: 0;
  margin: 5px;
}
.section {
  margin-bottom: 20px;
}
.section .title {
  background: #DDDDDD;
  border: 1px black solid;
  padding: 4px 8px;
}
.section .body {
  margin-left: 20px;
  padding: 10px;
}
#headerbar {
  background: #333333;
  border-bottom: 1px #DDDDDD solid;
  color: white;
  margin: 0;
  padding: 0.5em;
}
#headerbar br {
  clear: both;
  height: 0;
}
#headerbar #title {
  color: white;
  display: inline;
  font-weight: normal;
  font-size: 120%;
  vertical-align: middle;
  margin: 0;
  padding: 0;
}
#headerbar #title a {
  color: white;
  font-size: 120%;
  text-decoration: none;
}
#headerbar #title .version {
  color: #E0E0E0;
  font-weight: normal;
  font-size: 90%;
}
#headerbar #logo {
  position: absolute;
  top: -2px;
  left: 6px;
}
#headerbar #search {
  float: right;
  margin: 0 1em;
}
#headerbar #search input {
  border: 1px black solid;
  margin: 0;
  padding: 2px;
  -moz-border-radius: 6px;
  -webkit-border-radius: 6px;
  border-radius: 6px;
  background-color: #EEEEEE;
  font-size: 1.2em;
}
#rbinfo {
  float: left;
  position: absolute;
  left: 0;
  padding-top: 0.2em;
  padding-left: 72px;
}
#rbinfo img {
  display: inline;
  vertical-align: middle;
}
#accountnav {
  list-style: none;
  margin: 0;
  padding: 0;
  z-index: 1;
  float: right;
}
#accountnav li {
  display: block;
  float: right;
  margin: -0.5em 0;
  padding: 0;
  position: relative;
  white-space: nowrap;
}
#accountnav li:hover {
  background: #555555;
}
#accountnav li:hover ul {
  display: block;
  z-index: 110;
}
#accountnav li a {
  color: white;
  display: block;
  font-size: 120%;
  margin: 0;
  padding: 0 1em;
  text-decoration: none;
  height: 32px;
  vertical-align: middle;
  line-height: 32px;
}
#accountnav li a.user-nav-item {
  padding-left: 0.2em;
}
#accountnav li img {
  display: inline;
  vertical-align: top;
  margin: 0;
  padding: 0;
}
#accountnav li ul {
  background-color: #333333;
  display: none;
  margin: 0;
  min-width: 9em;
  padding: 0;
  position: absolute;
  right: 0;
  -moz-border-radius: 0 0 6px 6px;
  -webkit-border-radius: 0 0 6px 6px;
  border-radius: 0 0 6px 6px;
}
#accountnav li ul li {
  float: none;
  margin: 0;
  padding: 0;
  text-align: right;
}
#accountnav li ul li:last-child {
  -moz-border-radius: 0 0 6px 6px;
  -webkit-border-radius: 0 0 6px 6px;
  border-radius: 0 0 6px 6px;
}
#accountnav li ul li a {
  margin: 0;
}
#activity-indicator {
  position: fixed;
  top: 0;
  left: 50%;
  text-align: center;
  font-weight: bold;
  background-color: #fce94f;
  background-image: url("../images/spinner.gif");
  background-position: 0.4em 0.4em;
  background-repeat: no-repeat;
  border: 1px #c4a000 solid;
  border-top: 0;
  margin-left: -3em;
  padding: 0.5em 0.6em 0.5em 2.2em;
  width: 6em;
  z-index: 140;
}
#activity-indicator.error {
  background-color: #ffcd9c;
  width: 28em;
  margin-left: -14em;
  background-image: none;
  padding-left: 0.6em;
}
#activity-indicator.error a {
  margin-left: 1em;
  color: #0000CC;
  text-decoration: none;
}
#error div {
  background: #ffcd9c;
  border: 1px #9a4306 solid;
  margin-bottom: 10px;
  padding: 8px 10px;
}
#error h1 {
  display: inline;
  margin: 0;
  padding: 0;
}
#error input[type='submit'] {
  margin-left: 10px;
}
#manual-updates {
  padding-top: 1em;
}
#manual-updates .box-main {
  padding: 1em;
}
#manual-updates .box-main h1,
#manual-updates .box-main h2 {
  margin-top: 2em;
}
#manual-updates p {
  font-size: 9pt;
}
#navbar-container {
  margin: 0 0 1em 0;
}
#navbar {
  background: #c6dcf3;
  border-bottom: 1px #3465a4 solid;
  list-style: none;
  margin: 0;
  min-height: 1.5em;
  padding: 0.5em;
  padding-left: 72px;
}
#navbar li {
  display: inline;
  height: 30px;
  margin: 0 1em 0 0;
}
#navbar a {
  color: #0000aa;
  font-size: 110%;
  text-decoration: none;
}
#navbar .newreviewrequest {
  text-align: right;
}
body.admin #navbar {
  background: url("../images/admin/yellow_bg_small.png") repeat-x #f9f2c9;
  margin: 0;
}
body.admin #navbar a {
  color: black;
}
#page-container {
  padding: 0 1em 1em 1em;
}
#searchbar {
  position: absolute;
  right: 0;
  margin-top: -3px;
}
/****************************************************************************
 * Login/Register pages
 ****************************************************************************/
#auth_container {
  font-size: 120%;
  margin: 7em 0 0 0;
  text-align: center;
}
#auth_container input[type="text"],
#auth_container input[type="password"],
#auth_container input[type="email"] {
  border: 1px #aaa solid;
  font-size: inherit;
  margin: 0;
  padding: 0.5em;
  width: 16em;
  -moz-border-radius: 6px;
  -webkit-border-radius: 6px;
  border-radius: 6px;
  -moz-box-sizing: border-box;
  -webkit-box-sizing: border-box;
  box-sizing: border-box;
}
#auth_container .auth-button-container {
  width: 16em;
}
#auth_container .auth-button-container input {
  font-size: 120%;
  margin: 0;
  padding: 0.6em;
  width: 100%;
  -moz-box-sizing: border-box;
  -webkit-box-sizing: border-box;
  box-sizing: border-box;
}
#auth_container .auth-form-row {
  clear: both;
  margin: 1.5em 0;
  padding-left: 12em;
}
#auth_container .auth-form-row.auth-field-row {
  padding-left: 0;
}
#auth_container .auth-form-row.auth-field-row label {
  float: left;
  font-weight: normal;
  padding: 0.5em 1em 0.5em 0.5em;
  text-align: right;
  width: 12em;
  -moz-box-sizing: border-box;
  -webkit-box-sizing: border-box;
  box-sizing: border-box;
}
#auth_container .auth-form-row.auth-field-row .errorlist {
  display: block;
  margin: 0.5em 0 0 12em;
  width: 16em;
}
#auth_container .auth-form-row.auth-field-row .errorlist li {
  font-weight: normal;
  font-size: 9pt;
}
#auth_container .auth-header {
  margin: 0 0 1em 0;
}
#auth_container .auth-header h1 {
  font-size: 120%;
  margin: 1em 0;
  padding: 0;
}
#auth_container .auth-header p {
  color: #444;
  margin: 1em 0;
}
#auth_container .auth-header .errorbox {
  display: inline-block;
  text-align: center;
}
#auth_container .auth-header .errorbox .errorlist {
  text-align: left;
}
#auth_container .auth-section {
  display: inline-block;
  margin: 0 2em;
  text-align: left;
  vertical-align: top;
}
#auth_container .auth-section.main-auth-section {
  margin-left: -10em;
}
#auth_container .errorlist {
  margin: 0;
  padding: 0;
}
#auth_container .errorlist li {
  display: block;
  font-weight: normal;
  margin: 0 0 1em 0;
}
#auth_container #login_form .login-links {
  margin-top: 2em;
  text-align: center;
}
#auth_container #login_form .login-links p {
  width: 16em;
}
#auth_container #login_form .login-links p a {
  color: blue;
  text-decoration: none;
}
#auth_container #register_form .register-captcha-row {
  padding-left: 0;
}
#auth_container #register_form .register-captcha-row .register-captcha-container {
  float: right;
}
/****************************************************************************
 * Auto-complete widget
 ****************************************************************************/
.ui-autocomplete-results {
  background: #ffffff;
  border: 1px solid #808080;
  overflow: hidden;
  position: absolute;
  width: 100%;
  z-index: 110;
}
.ui-autocomplete-results ul {
  margin: 0;
  padding: 0;
  width: 100%;
}
.ui-autocomplete-results ul li {
  cursor: pointer;
  margin: 0;
  padding: 3px 6px;
  position: relative;
  white-space: nowrap;
}
.ui-autocomplete-results ul li span {
  margin-left: 6px;
  position: absolute;
  right: 6px;
}
.ui-autocomplete-results .ui-autocomplete-over {
  background: #71a5db;
  color: #FFF;
}
.ui-autocomplete-footer {
  background: #ECECEC;
  border-top: 1px #C0C0C0 solid;
  padding: 3px 6px;
}
/****************************************************************************
 * User page hover
 ****************************************************************************/
#user-infobox {
  background: #F9F9F9;
  border: 1px black solid;
  display: block;
  min-height: 12em;
  padding: 1em;
  position: absolute;
  min-width: 30em;
  z-index: 100;
  -moz-border-radius: 10px;
  -webkit-border-radius: 10px;
  border-radius: 10px;
  -moz-box-shadow: 0px 0px 4px #000000;
  -webkit-box-shadow: 0px 0px 4px #000000;
  box-shadow: 0px 0px 4px #000000;
}
#user-infobox.loading {
  background-image: url("../images/spinner.gif");
  background-position: 1em 1em;
  background-repeat: no-repeat;
}
#user-infobox .infobox-pic {
  float: left;
  width: 85px;
  margin-right: 1em;
}
#user-infobox .infobox-text {
  color: black;
  float: left;
  max-width: 200px;
  word-wrap: break-word;
}
#user-infobox a {
  text-decoration: none;
}
#user-infobox a:hover {
  text-decoration: underline;
}
#user-infobox p {
  font-size: 1.2em;
  margin: 0.5em 0;
  padding: 0;
}
#user-infobox h2 {
  font-size: 1.4em;
  margin: 0 0 0.5em 0;
  padding: 0;
}
#user-infobox h2 a {
  color: inherit;
}
#user-infobox .logged-in {
  margin-top: 2em;
}
#user-infobox .logged-in,
#user-infobox .joined {
  font-size: 0.8em;
}
#user-infobox #infobox-text h2 {
  margin-top: 0;
  padding-top: 0;
}
#submitter {
  position: relative;
}
/****************************************************************************
 * New Review Request
 ****************************************************************************/
.new-review-request {
  background-color: #fefadf;
  background-image: url('../images/review_request_box_top_bg.png');
}
.new-review-request label {
  color: #575012;
  font-weight: bold;
}
.new-review-request tr {
  padding-top: 4px;
}
.new-review-request .box-inner {
  background-image: url('../images/review_request_box_bottom_bg.png');
}
.new-review-request table,
#id_basedir,
#id_diff_path,
#id_parent_diff_path {
  width: 100%;
}
/****************************************************************************
 * clearfix hacks
 ****************************************************************************/
/*
 * clearfix hack. See http://www.webtoolkit.info/css-clearfix.html
 */
.clearfix {
  display: inline-block;
}
.clearfix:after {
  content: ".";
  display: block;
  clear: both;
  visibility: hidden;
  line-height: 0;
  height: 0;
}
html[xmlns] .clearfix {
  display: block;
}
* html .clearfix {
  height: 1%;
}
