{% extends "base.html" %}
{% load djblets_deco i18n %}

{% block title %}{% trans "Manual server updates required" %}{% endblock %}
{% block navbar %}{% endblock %}
{% block headerbar %}{% endblock %}

{% block content %}
<div id="manual-updates">
{% box "important" %}
 <h1>{% trans "Manual server updates required" %}</h1>
 <p>
{% blocktrans %}
  A recent upgrade requires manual updates to be made on this server. After
  these changes are made, you should restart your web server.
{% endblocktrans %}
 </p>
 <p>
{% blocktrans %}
  If you have any questions or problems, please contact us on our
  <a href="http://groups.google.com/group/reviewboard">mailing list</a>.
{% endblocktrans %}
 </p>
{% endbox %}
{% for update in updates %}
{% box %}
{{update}}
{% endbox %}
{% endfor %}
</div>

{% endblock %}
