{% load gravatars %}

<div class="infobox-content" class="vcard">
{% if siteconfig_settings.integration_gravatars %}
 <div class="infobox-pic" class="logo">
  {% gravatar requested_user 85 %}
 </div>
{% endif %}
 <div class="infobox-text">
  <h2 class="nickname"><a href="{% url 'user' requested_user %}">{{requested_user}}</a></h2>
{% if show_profile %}
  <p class="fn">{{requested_user.first_name}} {{requested_user.last_name}}</p>
{% if request.user.is_authenticated %}
  <p class="email"><a href="mailto:{{requested_user.email}}">{{requested_user.email}}</a></p>
{% endif %}
  <p class="logged-in">Last logged in {{requested_user.last_login|date:"F jS, Y"}}</p>
  <p class="joined">Joined {{requested_user.date_joined|date:"F jS, Y"}}</p>
{% endif %}
 </div>
</div>
