{% load i18n %}
<div class="box-head">
 <h1 class="box-title">{% blocktrans %}Install GNU {{binaryname}}{% endblocktrans %}</h1>
</div>
<div class="box-main">
 <p>
{% blocktrans %}
  GNU {{binaryname}} must be in your executable search path for the diff viewer
  to work. The current search path is:
{% endblocktrans %}
 </p>
 <p><tt>{{search_path}}</tt></p>
 <p>
{% if platform == "win32" %}
{%  blocktrans %}
  You can install GNU {{binaryname}} by downloading and running the
  <a href="http://gnuwin32.sourceforge.net/downlinks/patch.php">installer</a>.
  Then add the directory containing {{binaryname}} to the system PATH
  (Start &gt; Control Panel &gt; Advanced &gt; Environment Variables &gt;
   System Variables &gt; PATH).
{%  endblocktrans %}

{% elif platform == "darwin" %}
{%  blocktrans %}
  {{binaryname}} should be installed on your system by default. Make sure
  it's in the file path and accessible by your web server.
{%  endblocktrans %}

{% else %}
{%  blocktrans %}
  Most Linux/UNIX systems provide a native package containing {{binaryname}},
  which is usually installed by default. Check with your distribution for
  the proper package to install.

{%  endblocktrans %}
{% endif %}
 </p>
</div>
