/* RB Colors */
/*
 * At least on Chrome, using points instead of pixels does not reliably
 * result in identical sizings when there are elements positioned inside
 * a <pre>. Pixels fixes this.
 */
/* Diff styles */
/* !(*%!(&^ (see http://www.positioniseverything.net/easyclearing.html) */
.clearfix {
  display: inline-block;
}
.clearfix:after {
  clear: both;
  content: ".";
  display: block;
  height: 0;
  visibility: hidden;
}
/* Hides from IE-mac \*/
* html .clearfix {
  height: 1%;
}
.clearfix {
  display: block;
}
/* End hide from IE-mac */
/****************************************************************************
 * Review-specific macros
 ****************************************************************************/
/* Keep the label from shifting when the pencil icon is hidden/shown. */
/****************************************************************************
 * Actions
 ****************************************************************************/
.actions-container {
  background: #e5d7a8;
  border-bottom: 1px #957d2c solid;
  float: right;
  width: 100%;
  -moz-border-radius: 5px 5px 0 0;
  -webkit-border-radius: 5px 5px 0 0;
  border-radius: 5px 5px 0 0;
}
.actions-container > ul {
  min-height: 2em;
}
.actions {
  float: right;
  list-style: none;
  margin: 0;
  padding: 0;
  white-space: nowrap;
}
.actions a {
  color: black;
  cursor: pointer;
  display: block;
  text-decoration: none;
  padding: 6px 10px;
  border-left: 1px #957d2c solid;
}
.actions img {
  vertical-align: middle;
}
.actions li {
  float: left;
  margin: 0;
  padding: 0;
}
.actions li:hover {
  background-color: #d7c179;
}
.actions li:active {
  background-color: #ceb359;
}
.actions li.primary {
  background-color: #e5d7a8;
}
.actions li.primary:hover {
  background-color: #d7c179;
}
.actions li.primary:active {
  background-color: #ceb359;
}
.actions .menu {
  background: #ceb359;
  border: 1px #957d2c solid;
  float: none;
  list-style: none;
  margin: 0;
  padding: 0;
  position: absolute;
  z-index: 110;
  -moz-border-radius: 0 0 6px 6px;
  -webkit-border-radius: 0 0 6px 6px;
  border-radius: 0 0 6px 6px;
  -moz-box-shadow: 1px 1px 2px rgba(0, 0, 0, 0.15);
  -webkit-box-shadow: 1px 1px 2px rgba(0, 0, 0, 0.15);
  box-shadow: 1px 1px 2px rgba(0, 0, 0, 0.15);
}
.actions .menu li {
  background: #e5d7a8;
  border: 0;
  float: none;
  margin: 0;
  padding: 0;
}
.actions .menu li:last-child {
  -moz-border-radius: 0 0 6px 6px;
  -webkit-border-radius: 0 0 6px 6px;
  border-radius: 0 0 6px 6px;
}
.actions .menu li:hover {
  background-color: #d7c179;
}
.actions .menu li a {
  color: black;
  border: 0;
  display: block;
  margin: 0;
  padding: 8px 10px;
  text-decoration: none;
}
.actions > li:last-child {
  -moz-border-radius: 0 5px 0 0;
  -webkit-border-radius: 0 5px 0 0;
  border-radius: 0 5px 0 0;
}
/****************************************************************************
 * Code Mirror
 ****************************************************************************/
.CodeMirror {
  border: 1px #b8b5a0 solid;
  height: auto;
  margin: 5px 0 0 0;
}
.CodeMirror .CodeMirror-lines {
  padding: 10px;
}
.CodeMirror .CodeMirror-scroll {
  overflow-y: hidden;
  overflow-x: auto;
}
.CodeMirror pre {
  padding: 0;
}
.CodeMirror .cm-header {
  color: black;
}
.CodeMirror .cm-comment {
  color: #4444cc;
}
.CodeMirror .cm-variable-2 {
  color: black;
}
/****************************************************************************
 * Review Request Box
 ****************************************************************************/
.markdown-info {
  color: blue;
  font-size: 8pt;
  font-weight: normal;
  float: right;
  line-height: 25px;
  margin-right: 0.5em;
  text-decoration: none;
}
.markdown-info:hover {
  text-decoration: underline;
}
.review-request {
  /*
   * Any changes to this need to be reflected in _resizeLayout in
   * reviewRequestEditorView.js
   */
  background-color: #fefadf;
  background-image: url('../images/review_request_box_top_bg.png');
}
.review-request label {
  color: #575012;
  font-weight: bold;
}
.review-request noscript form {
  margin: 10px 5px 5px 5px;
  text-align: right;
}
.review-request textarea {
  border: 1px #b8b5a0 solid;
  height: 1.5em;
  overflow: hidden;
}
.review-request .box-inner {
  background-image: url('../images/review_request_box_bottom_bg.png');
}
.review-request .content {
  padding: 10px;
  position: relative;
}
.review-request .content.sliding {
  margin-top: -5px;
}
.review-request .content img {
  border: none;
}
.review-request .content .editable.loading {
  color: transparent;
  background: white url("../images/spinner.gif") no-repeat 10px 10px;
}
.review-request .content .field-container {
  position: relative;
}
.review-request .content .field-container > pre {
  white-space: pre-wrap;
  /* CSS 3 */
  white-space: -moz-pre-wrap;
  /* Mozilla, 1999+ */
  white-space: -pre-wrap;
  /* Opera 4-6 */
  white-space: -o-pre-wrap;
  /* Opera 7 */
  word-wrap: break-word;
  /* IE 5.5+ */
  background-color: #FFFFFF;
  border: 1px #b8b5a0 solid;
  margin: 5px 0 0 0;
  padding: 10px;
}
.review-request .content-divider {
  border-top: 1px #C2C1B0 solid;
}
.review-request .inline-editor-form.summary-editor {
  margin-top: -2px;
  margin-left: -3px;
}
.review-request .inline-editor-form input[type='text']:not(:focus) {
  -moz-box-shadow: 0px 0px 4px rgba(255, 0, 0, 0.75);
  -webkit-box-shadow: 0px 0px 4px rgba(255, 0, 0, 0.75);
  box-shadow: 0px 0px 4px rgba(255, 0, 0, 0.75);
}
.review-request .main {
  clear: both;
  padding: 0;
}
.review-request #required-field-legend {
  color: darkRed;
  font-weight: bold;
  padding: 10px;
}
.review-request .controls {
  list-style: none;
  margin: 0;
  padding: 0 8px 4px 8px;
  text-align: right;
}
.review-request .controls a {
  color: black;
  text-decoration: none;
}
.review-request .controls a:hover {
  color: #0000C0;
}
.review-request .controls li {
  display: inline;
  padding-left: 8px;
}
.review-request .controls .rb-icon {
  vertical-align: bottom;
}
.review-request .header {
  padding: 10px;
}
.review-request .star {
  float: left;
  margin-right: 10px;
}
.review-request #review_request_details {
  border-spacing: 0px;
  float: right;
  width: 300px;
  padding: 10px;
}
.review-request #review_request_details a {
  color: #0000ee;
  text-decoration: none;
}
.review-request #review_request_details th {
  color: #39350b;
  font-weight: bold;
  text-align: left;
  white-space: nowrap;
}
.review-request #review_request_details tbody td,
.review-request #review_request_details tbody th {
  padding: 0;
  vertical-align: top;
}
.review-request #review_request_details tbody th,
.review-request #review_request_details tbody td .field {
  padding-top: 3px;
}
.review-request #review_request_details tbody td {
  padding-left: 1em;
}
.review-request #review_request_details tbody td span {
  display: inline-block;
  margin-right: 4px;
  word-wrap: break-word;
}
.review-request #review_request_details tbody td .editicon {
  margin-left: 0;
}
.review-request #review_request_details tbody th {
  padding-bottom: 1.5em;
  padding-left: 1em;
  text-align: right;
  white-space: nowrap;
}
.review-request #review_request_details thead.first-fieldset th {
  padding-top: 0;
}
.review-request #review_request_details thead th {
  color: #575012;
  font-size: 120%;
  padding: 2em 0 1em 0;
}
.review-request #review_request_details #target_people .inactive {
  text-decoration: line-through;
}
.review-request #review_request_details .buttons {
  display: block;
  margin-bottom: 1em;
  padding: 6px 0 0 0;
}
.review-request #review_request_details .buttons input {
  font-size: 11px;
}
.review-request #review_request_extra {
  clear: both;
}
.review-request #review_request_extra h3 {
  color: #575012;
  font-size: 120%;
  margin: 0;
  padding: 0;
}
.review-request #review_request_extra label.textarea-label {
  display: block;
  min-height: 16px;
}
.review-request #review_request_extra label.textarea-label img {
  vertical-align: middle;
}
.review-request #review_request_main {
  margin-right: 300px;
  position: relative;
}
.review-request #review_request_main label {
  font-size: 120%;
}
.review-request #review-request-warning {
  background-color: #f5a9a9;
  border: 1px solid #ff0000;
  display: none;
  font-weight: bold;
  padding: 5px;
  text-align: center;
}
.review-request #diff_revision {
  font-weight: bold;
  text-align: right;
  vertical-align: bottom;
}
.review-request #status {
  color: #39350b;
  margin: 5px 0;
}
.review-request #summary_wrapper {
  line-height: 2.6em;
  padding: 0;
}
.review-request #summary_wrapper label,
.review-request #summary_wrapper #field_summary {
  font-size: 14px;
}
.review-request #summary_wrapper label {
  margin-right: 0.5em;
}
.review-request #summary_wrapper .rb-icon-edit {
  vertical-align: text-bottom;
}
.review-request #summary_wrapper #field_summary {
  display: inline;
}
.review-request #summary_wrapper #field_summary a {
  color: black;
  text-decoration: none;
}
.review-request .actions-container .star {
  margin: 5px 0 0 5px;
}
.review-request #updated_time,
.review-request #created_time {
  text-align: right;
  vertical-align: bottom;
}
.review-request .editicon {
  text-decoration: none;
}
.review-request .required-flag {
  color: darkred;
  font-size: 10px;
  font-weight: bold;
  padding: 0 2px;
}
#review_request_form input[type='text'],
#review_request_form textarea {
  width: 400px;
}
/****************************************************************************
 * Review UIs
 ****************************************************************************/
.review-ui-box.has-review-ui-box-content .actions-container {
  -moz-border-radius: 5px 5px 0 0;
  -webkit-border-radius: 5px 5px 0 0;
  border-radius: 5px 5px 0 0;
}
.review-ui-box.has-review-ui-box-content .actions-container .actions > li:last-child {
  -moz-border-radius: 0 5px 0 0;
  -webkit-border-radius: 0 5px 0 0;
  border-radius: 0 5px 0 0;
}
.review-ui-box .actions-container {
  -moz-border-radius: 5px;
  -webkit-border-radius: 5px;
  border-radius: 5px;
}
.review-ui-box .actions-container .actions > li:last-child {
  -moz-border-radius: 0 5px 5px 0;
  -webkit-border-radius: 0 5px 5px 0;
  border-radius: 0 5px 5px 0;
}
.review-ui-box .review-ui-box-content {
  clear: both;
  padding: 0;
}
/****************************************************************************
 * Reviews
 ****************************************************************************/
.box.collapsed .body {
  display: none;
}
.box.collapsed .banner {
  margin-bottom: 0;
}
.box .collapse-button {
  border: 1px #333333 solid;
  cursor: pointer;
  float: left;
  margin-right: 0.5em;
  margin-top: -1px;
  padding: 0;
}
.review .box {
  background-color: #e9edf5;
  background-image: url('../images/review_box_top_bg.png');
}
.review .banner {
  margin-top: 10px;
}
.review .banner.floating {
  border-top: 0;
  margin-top: 0;
}
.review .body {
  background-color: #FAFAFA;
  border: 1px #AAAAAA solid;
  clear: both;
  margin: 5px;
  padding: 10px;
}
.review .body pre {
  white-space: pre-wrap;
  /* CSS 3 */
  white-space: -moz-pre-wrap;
  /* Mozilla, 1999+ */
  white-space: -pre-wrap;
  /* Opera 4-6 */
  white-space: -o-pre-wrap;
  /* Opera 7 */
  word-wrap: break-word;
  /* IE 5.5+ */
}
.review .body .comment-text:target {
  background-color: #E7EDF3;
  border: 1px solid #3465a4;
  padding: 1em;
}
.review .body .body_top,
.review .body .body_bottom {
  margin: 0;
}
.review .box-inner {
  background-image: url('../images/review_box_bottom_bg.png') !important;
  min-height: 32px;
}
.review .controls {
  list-style: none;
  margin: 0;
  padding: 0 8px 4px 8px;
  text-align: right;
}
.review .controls a {
  text-decoration: none;
}
.review .controls li {
  display: inline;
}
.review .comments,
.review .review-comments {
  border: 0;
}
.review .comments dd,
.review .review-comments dd {
  margin-bottom: 20px;
}
.review .comments dt thead th,
.review .review-comments dt thead th {
  font-weight: normal;
}
.review .comments .filename a,
.review .review-comments .filename a {
  color: black;
  font-weight: bold;
}
.review .comments .file-attachment,
.review .review-comments .file-attachment {
  background: #F0F0F0;
  border: 1px #C0C0C0 solid;
  padding: 5px;
}
.review .comments .file-attachment .file-attachment-comment-header,
.review .review-comments .file-attachment .file-attachment-comment-header {
  font-size: 120%;
  white-space: nowrap;
}
.review .comments .file-attachment .file-attachment-comment-header a,
.review .review-comments .file-attachment .file-attachment-comment-header a {
  text-decoration: none;
}
.review .comments .file-attachment .file-attachment-comment-header img,
.review .review-comments .file-attachment .file-attachment-comment-header img {
  padding-right: 10px;
  vertical-align: middle;
}
.review .comments .file-attachment .file-attachment-comment-header .caption,
.review .review-comments .file-attachment .file-attachment-comment-header .caption {
  color: black;
  margin: 5px 0 0 32px;
}
.review .comments .file-attachment .file-attachment-comment-header .filename,
.review .review-comments .file-attachment .file-attachment-comment-header .filename {
  color: black;
  font-weight: bold;
  margin: 0;
  vertical-align: middle;
}
.review .comments .file-attachment .thumbnail,
.review .review-comments .file-attachment .thumbnail {
  margin-top: 0.5em;
}
.review .comments .screenshot,
.review .review-comments .screenshot {
  background: #F0F0F0;
  border: 1px #C0C0C0 solid;
  font-size: 120%;
}
.review .comments .screenshot img,
.review .review-comments .screenshot img {
  padding: 5px;
}
.review .comments .screenshot .filename,
.review .review-comments .screenshot .filename {
  display: block;
  margin: 0;
  padding: 5px 10px;
}
.review .comments .sidebyside th,
.review .review-comments .sidebyside th {
  /*
       * The line numbers aren't clickable in the comment fragments in reviews,
       * so show a standard cursor instead of a pointer cursor.
       */
  cursor: initial;
}
.review .diff-error tbody {
  border: 1px #f3da00 solid;
}
.review .header {
  padding: 2px 4px 4px 4px;
}
.review .header a {
  color: black;
  text-decoration: none;
}
.review .reviewer {
  float: left;
  font-weight: bold;
  padding-top: 2px;
}
.review .reviewer a {
  position: relative;
}
.review .posted_time {
  text-align: right;
  padding-top: 2px;
}
.review .reply-comments {
  margin: 20px 0 0 0;
  padding: 0;
}
.review .reply-comments > li {
  display: inline;
}
.review .reply-comments > li.draft dt label a {
  color: #4e9a06;
}
.review .reply-comments > li dl {
  border-left: 2px #D0D0D0 solid;
  /*
         * Set a border-bottom to prevent a bug in IE where, without a
         * border-bottom, the content of the dl will shift left by the
         * border-left size for every following item.
         */
  border-bottom: 1px #FAFAFA solid;
  margin: 0;
  padding: 5px;
}
.review .reply-comments > li dl dt label {
  display: block;
  line-height: 16px;
  vertical-align: top;
}
.review .reply-comments > li dl dt label a {
  color: #AB5603;
  font-size: 1.1em;
  font-weight: bold;
  text-decoration: none;
}
.review .reply-comments > li dl dt span {
  color: #487858;
  font-weight: normal;
}
.review .reply-comments > li dl dd {
  margin-left: 10px;
}
.review .shipit {
  background: #aeff60;
  border: 1px #888888 solid;
  font-weight: bold;
  left: -10px;
  margin-bottom: 8px;
  padding: 4px 8px;
  position: relative;
  width: 4.5em;
  top: -8px;
  -moz-border-radius: 6px 0 6px 0;
  -webkit-border-radius: 6px 0 6px 0;
  border-radius: 6px 0 6px 0;
}
.review .shipit.with-issues {
  border: 1px #363000 solid;
  color: #205003;
  width: 11em;
  background-color: #ffe303;
  background-repeat: no-repeat;
  background-image: -webkit-gradient(linear, 0 0, 0 100%, from(#fff49c), color-stop(25%, #fff49c), to(#ffe303));
  background-image: -webkit-linear-gradient(#fff49c, #fff49c 25%, #ffe303);
  background-image: -moz-linear-gradient(top, #fff49c, #fff49c 25%, #ffe303);
  background-image: -ms-linear-gradient(#fff49c, #fff49c 25%, #ffe303);
  background-image: -o-linear-gradient(#fff49c, #fff49c 25%, #ffe303);
  background-image: linear-gradient(#fff49c, #fff49c 25%, #ffe303);
  filter: progid:DXImageTransform.Microsoft.gradient(startColorstr='#fff49c', endColorstr='#ffe303', GradientType=0);
}
.sidebyside.loading {
  /* Helps us better keep the size, as long as the comment is within a chunk. */
  padding: 2px 2px 1px 2px;
  width: 100%;
}
.sidebyside.loading tbody {
  background: #f3f3f3 url("../images/spinner.gif") no-repeat center center;
}
.sidebyside.loading tbody td {
  padding: 1em;
}
/****************************************************************************
 * Change Descriptions
 ****************************************************************************/
#field_changedescription {
  white-space: pre-wrap;
  /* CSS 3 */
  white-space: -moz-pre-wrap;
  /* Mozilla, 1999+ */
  white-space: -pre-wrap;
  /* Opera 4-6 */
  white-space: -o-pre-wrap;
  /* Opera 7 */
  word-wrap: break-word;
  /* IE 5.5+ */
  background-color: white;
  border: 1px #b8b5a0 solid;
  margin: 5px 0 0 0;
  padding: 10px;
}
#field_changedescription.loading {
  color: transparent;
  background: white url("../images/spinner.gif") no-repeat 10px 10px;
}
.changedesc.box {
  background-color: #fefadf;
  background-image: url('../images/review_request_box_top_bg.png');
}
.changedesc .body {
  clear: both;
  padding: 10px;
  /* Field labels */
  /* Used in the textarea field diffs and the Change Summary section. */
  /*
     * File attachment/screenshot change lists.
     *
     * These differ from the other field change lists in that they have
     * mini-section labels underneath for each file.
     */
  /* Displays changes for field values, with "-" and "+" markers. */
  /* The "Change Summary" section. */
  /* The diff for text areas. */
  /* Styling for the "Diff" field updates. */
  /* Styles for the file listing for the "Diff" field. */
  /*
     * Secondary fields are in more of a Field: Value display, with the
     * fields lining up.
     */
}
.changedesc .body > ul {
  list-style: none;
  margin: 1em 0 1em 0;
  padding: 0;
}
.changedesc .body > ul > li {
  margin: 3em 0 0 0;
}
.changedesc .body > ul > li:first-child {
  margin-top: 0;
}
.changedesc .body a {
  color: default;
  text-decoration: none;
}
.changedesc .body a:hover {
  text-decoration: underline;
}
.changedesc .body h3 {
  color: #575012;
  font-size: 110%;
  margin: 1em 0 0.5em 0;
  padding: 0;
}
.changedesc .body h3.status {
  /* Add some separation between this and the other fields. */
  margin-bottom: 2em;
}
.changedesc .body h3.status:last-child {
  /*
           * If it's the last child, we don't want that separation, or it'll
           * have an odd amount of extra space.
           */
  margin-bottom: 0.5em;
}
.changedesc .body h3.status .value {
  color: black;
  font-weight: normal;
}
.changedesc .body > pre {
  background-color: #FFFFFF;
  border: 1px #b8b5a0 solid;
  padding: 10px;
  font-size: 9pt;
  white-space: pre-wrap;
  /* CSS 3 */
  white-space: -moz-pre-wrap;
  /* Mozilla, 1999+ */
  white-space: -pre-wrap;
  /* Opera 4-6 */
  white-space: -o-pre-wrap;
  /* Opera 7 */
  word-wrap: break-word;
  /* IE 5.5+ */
}
.changedesc .body .caption-changed {
  margin: 2em 0 0 2em;
  padding: 0;
}
.changedesc .body .caption-changed a {
  /* Treat the link like a field. */
  font-weight: bold;
}
.changedesc .body .caption-changed td,
.changedesc .body .caption-changed th {
  vertical-align: top;
}
.changedesc .body .caption-changed td {
  padding-left: 1em;
}
.changedesc .body .caption-changed th {
  text-align: right;
}
.changedesc .body .changed {
  border: 1px #b8b5a0 solid;
  border-collapse: collapse;
  display: inline-block;
  margin: 0;
  padding: 0;
  vertical-align: top;
}
.changedesc .body .changed a {
  color: blue;
  text-decoration: none;
}
.changedesc .body .changed a:first-child {
  margin-left: 0;
}
.changedesc .body .changed a:hover {
  text-decoration: underline;
}
.changedesc .body .changed td,
.changedesc .body .changed th {
  font-family: monospace;
  padding: 0.3em 0.6em;
}
.changedesc .body .changed td {
  background: #FEFEFE;
}
.changedesc .body .changed .new-value .marker {
  background: #caf2c1;
}
.changedesc .body .changed .old-value .marker {
  background: #f5d0d6;
}
.changedesc .body .changedesc-text {
  margin-bottom: 2em;
}
.changedesc .body .changedesc-text:last-child {
  margin-bottom: 0;
}
.changedesc .body .diffed-text-area {
  background: white;
  border: 1px #b8b5a0 solid;
  border-collapse: collapse;
  padding: 0;
  width: 100%;
  /* The '-' or '+' marker. */
}
.changedesc .body .diffed-text-area a {
  text-decoration: underline;
}
.changedesc .body .diffed-text-area pre {
  background: none;
  border: 0;
  margin: 0;
  padding: 0;
  white-space: pre-wrap;
  /* CSS 3 */
  white-space: -moz-pre-wrap;
  /* Mozilla, 1999+ */
  white-space: -pre-wrap;
  /* Opera 4-6 */
  white-space: -o-pre-wrap;
  /* Opera 7 */
  word-wrap: break-word;
  /* IE 5.5+ */
}
.changedesc .body .diffed-text-area td {
  font-family: monospace;
  padding: 2px 4px;
  vertical-align: top;
  white-space: pre-wrap;
  /* CSS 3 */
  white-space: -moz-pre-wrap;
  /* Mozilla, 1999+ */
  white-space: -pre-wrap;
  /* Opera 4-6 */
  white-space: -o-pre-wrap;
  /* Opera 7 */
  word-wrap: break-word;
  /* IE 5.5+ */
}
.changedesc .body .diffed-text-area .delete {
  /* The '-' or '+' marker. */
}
.changedesc .body .diffed-text-area .delete .line {
  background: #ffe0e5;
}
.changedesc .body .diffed-text-area .delete .marker {
  color: #990000;
  background: #f5d0d6;
}
.changedesc .body .diffed-text-area .insert {
  /* The '-' or '+' marker. */
}
.changedesc .body .diffed-text-area .insert .line {
  background: #dfffd7;
}
.changedesc .body .diffed-text-area .insert .marker {
  background: #caf2c1;
}
.changedesc .body .diffed-text-area .line {
  width: 100%;
  /*
         * Make sure any images in the Markdown-rendered text are kept
         * small, but with the correct aspect ratio.
         */
}
.changedesc .body .diffed-text-area .line img {
  width: auto;
  max-height: 100px;
}
.changedesc .body .diffed-text-area .marker {
  background: #F9F9F9;
  font-family: monospace;
  font-size: 13px;
  font-weight: bold;
  text-align: center;
}
.changedesc .body .diffed-text-area .replace-new {
  /* The '-' or '+' marker. */
}
.changedesc .body .diffed-text-area .replace-new .line {
  background: #dfffd7;
}
.changedesc .body .diffed-text-area .replace-new .line .hl {
  background: #acee9c;
}
.changedesc .body .diffed-text-area .replace-new .marker {
  background: #caf2c1;
}
.changedesc .body .diffed-text-area .replace-old {
  /* The '-' or '+' marker. */
}
.changedesc .body .diffed-text-area .replace-old .line {
  background: #ffe0e5;
}
.changedesc .body .diffed-text-area .replace-old .line .hl {
  background: #f7b5c0;
}
.changedesc .body .diffed-text-area .replace-old .marker {
  background: #f5d0d6;
}
.changedesc .body .diff-changes .line-counts {
  margin-left: 0.5em;
}
.changedesc .body .diff-changes .line-counts .delete-count {
  color: darkred;
}
.changedesc .body .diff-changes .line-counts .insert-count {
  color: darkgreen;
}
.changedesc .body .diff-index {
  margin-bottom: 0;
}
.changedesc .body .diff-index .diff-file-icon {
  min-width: 20px;
  min-height: 20px;
}
.changedesc .body .diff-index .diff-file-info {
  /* Compensate for the lack of diff-chunks-cell column. */
  width: 100%;
}
.changedesc .body .primary-fields h3:first-child {
  margin-top: 2em;
}
.changedesc .body .secondary-fields {
  width: 100%;
}
.changedesc .body .secondary-fields h3 {
  margin: 2px 1em 0 0;
  padding: 0;
  text-align: right;
}
.changedesc .body .secondary-fields > tbody > tr > td,
.changedesc .body .secondary-fields > tbody > tr > th {
  padding: 0.5em 0;
  vertical-align: top;
  white-space: nowrap;
}
.changedesc .body .secondary-fields > tbody > tr > td {
  /*
           * Between this and the white-space: nowrap above, the field labels
           * will end up taking the minimum size without wrapping, and the
           * rest of the width will be given to the value.
           */
  width: 100%;
}
.changedesc .body .secondary-fields > tbody > tr > td > p {
  /*
             * If the field value is rendered inside a <p>, make sure it
             * aligns with the field label.
             *
             * Note that the label has a larger font than this, so we can't
             * reuse the same pixel value. We have to bump it.
             */
  /* Don't take up more room than we need in the last <p>. */
}
.changedesc .body .secondary-fields > tbody > tr > td > p:first-child {
  margin-top: 3px;
}
.changedesc .body .secondary-fields > tbody > tr > td > p:last-child {
  margin-bottom: 0;
}
.changedesc .box-inner {
  background-image: url('../images/review_request_box_bottom_bg.png') !important;
  min-height: 32px;
}
.changedesc .header {
  padding: 2px 4px 4px 4px;
}
.changedesc .header a {
  color: black;
  text-decoration: none;
}
.changedesc .header b {
  float: left;
  font-weight: bold;
}
.changedesc .header .reviewer {
  padding-top: 2px;
}
.changedesc .header .posted_time {
  text-align: right;
}
/****************************************************************************
 * Updates Bubble
 ****************************************************************************/
#updates-bubble {
  background: #FFF1AB;
  border: 1px #666666 solid;
  bottom: 1px;
  font-size: 110%;
  padding: 0.8em;
  position: fixed;
  right: 1px;
}
#updates-bubble a,
#updates-bubble a:visited {
  color: #0000CC;
  text-decoration: none;
}
#updates-bubble #updates-bubble-buttons {
  margin-left: 2em;
}
/****************************************************************************
 * Review Form
 ****************************************************************************/
#review-form {
  overflow: auto;
  padding: 10px;
}
#review-form label {
  color: black;
  font-weight: bold;
  margin-top: 10px;
}
#review-form .edit-field {
  margin: 1em 0;
}
#review-form .edit-field .comment-text-field label {
  display: block;
  line-height: 16px;
  vertical-align: top;
}
#review-form .spinner {
  background: url("../images/spinner.gif") no-repeat center center;
  height: 16px;
  margin: 3em auto;
}
#review-form #review-form-comments {
  background: none;
}
#review-form #review-form-comments .comments {
  list-style: none;
  margin: 0;
  padding: 0;
}
#review-form #review-form-comments .comments li {
  margin: 2em 0;
  padding: 0;
}
#review-form #review-form-comments .comments li .edit-fields {
  margin-left: 4em;
}
/****************************************************************************
 * Uploaded files
 ****************************************************************************/
.file-container.editing .file {
  -moz-border-radius: 6px 6px 0 0;
  -webkit-border-radius: 6px 6px 0 0;
  border-radius: 6px 6px 0 0;
}
.file-container.editing .file .file-caption-container {
  padding-left: 0;
  padding-right: 0;
}
.file-container.editing .file .file-caption-container .file-caption {
  text-overflow: initial;
}
.file-container .file {
  background-color: #FEFEFE;
  border: 1px #cccccc solid;
  display: block;
  float: left;
  position: relative;
  margin: 0 1.5em 1.5em 0;
  max-width: 300px;
  width: 300px;
  -moz-border-radius: 6px;
  -webkit-border-radius: 6px;
  border-radius: 6px;
  -moz-box-shadow: 1px 1px 2px rgba(0, 0, 0, 0.15);
  -webkit-box-shadow: 1px 1px 2px rgba(0, 0, 0, 0.15);
  box-shadow: 1px 1px 2px rgba(0, 0, 0, 0.15);
  -moz-box-sizing: border-box;
  -webkit-box-sizing: border-box;
  box-sizing: border-box;
}
.file-container .file .actions li {
  line-height: 16px;
}
.file-container .file .actions li:first-child a {
  -moz-border-radius: 0 0 0 5px;
  -webkit-border-radius: 0 0 0 5px;
  border-radius: 0 0 0 5px;
}
.file-container .file .actions li:last-child a {
  -moz-border-radius: 0 5px 0 0;
  -webkit-border-radius: 0 5px 0 0;
  border-radius: 0 5px 0 0;
}
.file-container .file .actions li:first-child:last-child a {
  -moz-border-radius: 0 5px 0 5px;
  -webkit-border-radius: 0 5px 0 5px;
  border-radius: 0 5px 0 5px;
}
.file-container .file .actions li:hover {
  background: transparent;
}
.file-container .file .actions li a {
  border-left: 1px #DDD solid;
  border-bottom: 1px #DDD solid;
  color: black;
  padding: 6px 8px;
}
.file-container .file .actions li a:hover {
  background: #EEEEEE;
  text-decoration: none;
}
.file-container .file .file-header {
  background: white;
  padding: 0 0.5em;
  line-height: 2.5em;
  height: 2.5em;
  -moz-border-radius: 6px 6px 0 0;
  -webkit-border-radius: 6px 6px 0 0;
  border-radius: 6px 6px 0 0;
  /* Ellipsize the text contents. */
  overflow: hidden;
  white-space: nowrap;
  text-overflow: ellipsis;
}
.file-container .file .file-header a {
  color: blue;
  font-weight: bold;
  text-decoration: none;
}
.file-container .file .file-header img {
  vertical-align: middle;
}
.file-container .file .file-thumbnail-container {
  background: white;
  height: 100px;
  padding: 1em;
  text-align: center;
}
.file-container .file .file-thumbnail-container a {
  color: black;
  text-decoration: none;
}
.file-container .file .file-thumbnail-container pre {
  font-size: x-small;
}
.file-container .file .file-thumbnail-container .file-thumbnail-overlay {
  max-height: 100px;
  max-width: 100%;
  margin: 0;
  padding: 0;
  position: absolute;
  left: 0;
  width: 100%;
  height: 100%;
  z-index: 30;
}
.file-container .file .file-thumbnail-container .file-thumbnail {
  max-height: 100px;
  max-width: 100%;
  margin: 0;
  padding: 0;
  border: 0;
  overflow: hidden;
  text-align: left;
  white-space: nowrap;
  vertical-align: middle;
}
.file-container .file .file-thumbnail-container .file-thumbnail-clipped {
  max-height: 100px;
  max-width: 100%;
  margin: 0;
  padding: 0;
  border: 0;
  overflow: hidden;
  text-align: left;
  font-size: 8pt;
  /* Override the styling used for the <pre>s for fields. */
}
.file-container .file .file-thumbnail-container .file-thumbnail-clipped pre {
  background-color: transparent;
  border: 0;
  margin: 0;
  padding: 0;
}
.file-container .file .file-caption-container {
  border-top: 1px #EEEEEE solid;
  line-height: 2.5em;
  padding: 0 30px;
  position: relative;
  text-align: center;
  -moz-border-radius: 0 0 7px 7px;
  -webkit-border-radius: 0 0 7px 7px;
  border-radius: 0 0 7px 7px;
}
.file-container .file .file-caption-container a {
  color: black;
  text-decoration: none;
}
.file-container .file .file-caption-container a.empty-caption {
  color: gray;
  font-style: italic;
}
.file-container .file .file-caption-container .file-caption {
  line-height: 31px;
  height: 31px;
  /* Ellipsize the text contents. */
  overflow: hidden;
  white-space: nowrap;
  text-overflow: ellipsis;
}
.file-container .file .file-caption-container .file-caption .buttons {
  /*
           * This provides enough padding to cleanly cover much of any file
           * attachment's header below.
           */
  position: absolute;
  padding: 5px 0;
  /* Line up with .file's side borders. */
  margin: 0 0 0 -1px;
  text-align: center;
  background: #f2f2f2;
  border: 1px #cccccc solid;
  border-top: 0;
  display: block;
  width: 300px;
  -moz-border-radius: 0 0 6px 6px;
  -webkit-border-radius: 0 0 6px 6px;
  border-radius: 0 0 6px 6px;
  -moz-box-sizing: border-box;
  -webkit-box-sizing: border-box;
  box-sizing: border-box;
  -moz-box-shadow: 1px 1px 2px rgba(0, 0, 0, 0.15);
  -webkit-box-shadow: 1px 1px 2px rgba(0, 0, 0, 0.15);
  box-shadow: 1px 1px 2px rgba(0, 0, 0, 0.15);
  z-index: 20;
}
.file-container .file .file-caption-container .inline-editor-form > input[type='text'] {
  background: transparent;
  border: 0;
  font-family: inherit;
  font-size: inherit;
  height: 31px;
  margin: 0;
  padding-top: 0;
  padding-bottom: 0;
  /*
         * The 4px is needed to align the input with the text. It matches
         * the margin between the text and pencil.
         */
  padding: 0 4px 0 0;
  text-align: inherit;
  vertical-align: bottom;
  -moz-box-sizing: border-box;
  -webkit-box-sizing: border-box;
  box-sizing: border-box;
  /*
         * The box shadow doesn't display correctly in file attachment
         * thumbnails, so hide it. There will be a red background instead.
         */
}
.file-container .file .file-caption-container .inline-editor-form > input[type='text']:focus {
  outline: 0;
}
.file-container .file .file-caption-container .inline-editor-form > input[type='text']:not(:focus) {
  background: rgba(255, 0, 0, 0.15);
  -moz-box-shadow: none;
  -webkit-box-shadow: none;
  box-shadow: none;
}
.file-container .file .file-caption-container .inline-editor-form > input[type='text']::-ms-clear {
  display: none;
}
.file-container .file .file-caption-container .rb-icon-edit {
  vertical-align: middle;
  position: absolute;
  right: 6px;
  bottom: 6px;
}
/****************************************************************************
 * Screenshots
 ****************************************************************************/
.screenshot-caption {
  background-color: #F6F6F6;
  border: 1px #999999 solid;
  border-top: 1px #CCCCCC solid;
  padding: 2px 8px;
  -moz-border-radius: 0 0 6px 6px;
  -webkit-border-radius: 0 0 6px 6px;
  border-radius: 0 0 6px 6px;
}
.screenshot-caption img {
  vertical-align: bottom;
}
.screenshot-container {
  background: white;
  float: left;
  display: inline;
  margin-bottom: 10px;
  margin-right: 10px;
  max-width: 400px;
  min-width: 130px;
  text-align: center;
  vertical-align: middle;
  position: relative;
  -moz-border-radius: 6px;
  -webkit-border-radius: 6px;
  border-radius: 6px;
}
.screenshot-container:hover {
  background-color: #F6F6F6;
}
.screenshot-container:hover .screenshot-caption {
  background-color: #E6E6E6;
}
.screenshot-container a {
  color: black;
  text-decoration: none;
}
.screenshot-container .image {
  border: 1px #999999 solid;
  border-bottom: 0;
  cursor: pointer;
  height: 100px;
  line-height: 100px;
  margin: 0 auto;
  vertical-align: bottom;
  -moz-border-radius: 6px 6px 0 0;
  -webkit-border-radius: 6px 6px 0 0;
  border-radius: 6px 6px 0 0;
}
.screenshot-container .image.loading {
  background: white url("../images/spinner.gif") no-repeat center center;
}
.screenshot-container .image img {
  vertical-align: middle;
  -moz-border-radius: 5px 5px 0 0;
  -webkit-border-radius: 5px 5px 0 0;
  border-radius: 5px 5px 0 0;
}
/****************************************************************************
 * Screenshot Thumbnails
 ****************************************************************************/
#screenshot-thumbnails {
  margin: 2px;
  padding: 0.5em;
}
#screenshot-thumbnails.dragover {
  border: 2px green dashed;
  display: block;
  margin: 0;
}
/****************************************************************************
 * List of Attached Files
 ****************************************************************************/
#file-list {
  margin: 2px;
  padding: 1em 0.5em 0.5em 0.5em;
}
#file-list.dragover {
  border: 2px green dashed;
  display: block;
  margin: 0;
}
#file-list h1.drop-indicator {
  clear: both;
  text-align: center;
}
/****************************************************************************
 * Comment detail dialog
 ****************************************************************************/
.comment-dlg {
  border: 1px black solid;
  cursor: default;
  margin-top: 8px;
  position: absolute;
}
.comment-dlg form {
  background-color: #aceb6f;
  display: block;
  padding: 6px;
}
.comment-dlg form .buttons {
  bottom: 6px;
  left: 6px;
  margin-top: 6px;
  position: absolute;
}
.comment-dlg form .buttons input[type="button"] {
  margin-right: 3px;
}
.comment-dlg h1 {
  background: transparent;
  border: 0;
  margin: 0 0 6px 0;
  padding: 0;
}
.comment-dlg img.ui-resizable-grip {
  /* Use -1 for bottom and right so it overlaps the resize borders. */
  bottom: -1px;
  cursor: se-resize;
  padding: 0.5em;
  position: absolute;
  right: -1px;
  z-index: 20;
}
.comment-dlg .markdown-info {
  line-height: 16px;
}
.comment-dlg .status {
  margin: 4px 0;
  font-style: italic;
}
.comment-dlg .other-comments {
  background-color: #a1c0e3;
  border-right: 1px black solid;
  padding: 6px;
}
.comment-dlg .other-comments ul {
  background-color: white;
  border: 1px #333333 solid;
  list-style: none;
  margin: 0;
  overflow: auto;
  padding: 0;
  position: relative;
}
.comment-dlg .other-comments ul li {
  padding: 6px;
}
.comment-dlg .other-comments ul li.even {
  background-color: #F3F3F3;
}
.comment-dlg .other-comments ul li h2 {
  font-size: 90%;
  margin: 0 0 6px 0;
  padding: 0;
}
.comment-dlg .other-comments ul li h2 .actions {
  font-weight: normal;
  position: absolute;
  right: 6px;
}
.comment-dlg .other-comments ul li h2 .actions a {
  background-color: #EEEEEE;
  border: 1px black outset;
  color: black;
  cursor: pointer;
  text-decoration: none;
  margin-left: 6px;
  padding: 1px 3px;
}
.comment-dlg .other-comments ul li h2 .actions a:hover {
  background-color: #F6F6F6;
}
.comment-dlg .other-comments ul li pre {
  white-space: pre-wrap;
  /* CSS 3 */
  white-space: -moz-pre-wrap;
  /* Mozilla, 1999+ */
  white-space: -pre-wrap;
  /* Opera 4-6 */
  white-space: -o-pre-wrap;
  /* Opera 7 */
  word-wrap: break-word;
  /* IE 5.5+ */
  font-size: 90%;
}
.comment-dlg .ui-icon-gripsmall-diagonal-se {
  background: url('../images/resize-grip.png') no-repeat top left;
}
.issue-state {
  font-weight: bold;
  line-height: 26px;
  min-height: 26px;
  padding: 0.3em 0.6em 0.3em 0.6em;
  position: relative;
}
.issue-state input,
.issue-state .back-to-issue-summary {
  /*
     * This keeps a consistency with the browser. By default, Chrome uses
     * a margin of 2px, and changing it to 0 actually makes things look
     * off-center.
     */
  margin: 2px 2px 2px 0.2em;
  vertical-align: middle;
}
.issue-state .issue-container {
  position: relative;
}
.issue-state .rb-icon {
  margin: 0 0.2em;
}
.issue-state.dropped {
  background: #ececec;
  border: 1px solid #b9b9b9;
}
.issue-state.dropped .back-to-issue-summary,
.issue-state.dropped .back-to-issue-summary:visited {
  color: #535353;
}
.issue-state.open {
  background: #fff49c;
  border: 1px solid #cfb800;
}
.issue-state.open .back-to-issue-summary,
.issue-state.open .back-to-issue-summary:visited {
  color: #695d00;
}
.issue-state.resolved {
  background: #fafafa;
  border: 1px solid #e1e1e1;
}
.issue-state.resolved .back-to-issue-summary,
.issue-state.resolved .back-to-issue-summary:visited {
  color: #616161;
}
.issue-state .back-to-issue-summary {
  font-size: 13px;
  font-weight: normal;
  text-decoration: none;
  padding: 0 0.5em;
  position: absolute;
  right: 0;
  height: 24px;
  line-height: 24px;
}
.issue-state .back-to-issue-summary:hover {
  text-decoration: underline;
}
.issue-state .issue-message {
  margin-right: 0.5em;
  vertical-align: middle;
}
/****************************************************************************
 * Banners
 ****************************************************************************/
#discard-banner,
#submitted-banner {
  background: #E0E0E0;
  border: 1px #808080 solid;
  -moz-border-radius: 6px;
  -webkit-border-radius: 6px;
  border-radius: 6px;
  -moz-box-shadow: 1px 1px 2px rgba(0, 0, 0, 0.15);
  -webkit-box-shadow: 1px 1px 2px rgba(0, 0, 0, 0.15);
  box-shadow: 1px 1px 2px rgba(0, 0, 0, 0.15);
}
#draft-banner {
  -moz-border-radius: 6px;
  -webkit-border-radius: 6px;
  border-radius: 6px;
  -moz-box-shadow: 1px 1px 2px rgba(0, 0, 0, 0.15);
  -webkit-box-shadow: 1px 1px 2px rgba(0, 0, 0, 0.15);
  box-shadow: 1px 1px 2px rgba(0, 0, 0, 0.15);
}
#draft-banner p label,
#discard-banner p label,
#submitted-banner p label {
  display: block;
  line-height: 16px;
  vertical-align: top;
}
#draft-banner > p,
#discard-banner > p,
#submitted-banner > p {
  margin: 1em 0 0 0;
}
#review-banner {
  padding-bottom: 4em;
}
#review-banner .banner {
  border-left: 0;
  border-right: 0;
  border-top: 0;
  left: 0;
  position: fixed;
  top: 0;
  width: 100%;
  z-index: 90;
  -moz-box-shadow: 1px 1px 2px rgba(0, 0, 0, 0.15);
  -webkit-box-shadow: 1px 1px 2px rgba(0, 0, 0, 0.15);
  box-shadow: 1px 1px 2px rgba(0, 0, 0, 0.15);
}
/****************************************************************************
 * Tooltips
 ****************************************************************************/
.tooltip {
  background-color: #fef6b3;
  border: 1px #a58600 solid;
  padding: 4px;
  position: absolute;
}
.tooltip.comments {
  border: 1px black solid;
  border-bottom: 0;
  padding: 0;
}
.tooltip.comments ul {
  list-style: none;
  margin: 0;
  padding: 0;
}
.tooltip.comments ul li {
  background-color: #a1c0e3;
  border-bottom: 1px #5170b3 solid;
  margin: 0;
  padding: 5px;
}
.tooltip.comments ul li.draft {
  background-color: #aceb6f;
  border-bottom: 1px black solid;
}
.tooltip.comments ul li pre {
  white-space: pre-wrap;
  /* CSS 3 */
  white-space: -moz-pre-wrap;
  /* Mozilla, 1999+ */
  white-space: -pre-wrap;
  /* Opera 4-6 */
  white-space: -o-pre-wrap;
  /* Opera 7 */
  word-wrap: break-word;
  /* IE 5.5+ */
  font-size: 90%;
  margin: 0;
  padding: 0;
}
/****************************************************************************
 * Bubbles
 ****************************************************************************/
.bubble {
  background-color: white;
  border: 1px black solid;
  font-size: 10pt;
  font-weight: bold;
  padding: 6px;
  position: absolute;
  white-space: nowrap;
  z-index: 100;
  -moz-border-radius: 8px;
  -webkit-border-radius: 8px;
  border-radius: 8px;
}
/****************************************************************************
 * Issue Summary
 ****************************************************************************/
.issue-table-wrapper {
  clear: both;
  margin: 8px 9px 14px 9px;
}
#issue-summary-table {
  border-spacing: 0;
  width: 100%;
}
#issue-summary-table tr td {
  border-width: 0 0 1px 0;
  border-style: solid;
  padding: 5px 5px;
  vertical-align: top;
}
#issue-summary-table tr td a {
  color: #0000CC;
  text-decoration: none;
}
#issue-summary-table tr.hidden {
  display: none;
}
#issue-summary-table thead th {
  background: url("../../djblets/images/datagrid/header_bg.png") repeat-x bottom left;
  border-color: #999;
  border-style: solid;
  border-width: 1px 0;
  cursor: pointer;
  height: 21px;
  padding: 2px 5px;
  text-align: left;
  white-space: nowrap;
}
#issue-summary-table thead th:hover {
  background: url("../../djblets/images/datagrid/header_bg_primary.png") repeat-x bottom left;
}
#issue-summary-table thead .description-header {
  border-left-width: 1px;
}
#issue-summary-table thead .status-header {
  border-right-width: 1px;
}
#issue-summary-table .issue .status {
  border-right-width: 1px;
}
#issue-summary-table .issue .summary-table-description {
  border-left-width: 1px;
  width: 100%;
}
#issue-summary-table .issue.open td {
  background-color: #FDF4A4;
  border-color: #a89c00;
}
#issue-summary-table .issue.dropped td {
  background-color: #eaeaea;
  border-color: #AAA;
}
#issue-summary-table .issue.resolved td {
  background-color: #FFF;
  border-color: #AAA;
}
#issue-summary-table .issue:hover td {
  background-color: #D4E0F3;
  border-color: #3465a4;
}
#issue-summary-table .issue .summary-anchor {
  color: #000;
  display: block;
  overflow: hidden;
  text-decoration: none;
}
#issue-summary-table .no-issues td {
  background-color: #FFF;
  border-color: #999;
  border-width: 1px;
}
#issue-summary-table .status {
  text-transform: capitalize;
}
#issue-summary-table .comment-id,
#issue-summary-table .last-updated,
#issue-summary-table .reviewer,
#issue-summary-table .status {
  white-space: nowrap;
}
#issue-summary .header {
  padding-bottom: 0;
  position: relative;
}
#issue-summary .header .counters {
  position: absolute;
  bottom: 0;
  right: 10px;
}
#issue-summary .header .counters .counter {
  margin-left: 4px;
}
#issue-summary .header .counters .counter.total label,
#issue-summary .header .counters .counter.total span {
  font-weight: bold;
}
#issue-summary .header .counters .counter label {
  color: black;
  font-weight: normal;
  margin-right: 3px;
}
#issue-summary .header .filters {
  margin: 3px 0 0 0;
}
#issue-summary .header .filters .filter-wrapper {
  display: inline;
  margin: 0 4px;
}
/****************************************************************************
 * Rich text
 ****************************************************************************/
/*
 * Note that there's similar styling code in
 * reviewboard/notifications/templatetags/markdown_email.py. If you make
 * substantial changes here, you may also want to make changes there.
 */
.rich-text {
  line-height: 15px;
  -moz-box-sizing: border-box;
  -webkit-box-sizing: border-box;
  box-sizing: border-box;
}
.rich-text * {
  margin: 0;
  padding: 0;
  line-height: inherit;
  text-rendering: inherit;
  white-space: normal;
}
.rich-text * br {
  display: block;
}
.rich-text h1,
.rich-text h2,
.rich-text h3,
.rich-text h4,
.rich-text h5,
.rich-text h6 {
  font-size: 100%;
}
.rich-text p {
  white-space: inherit;
  /* Don't show both the <br> and the blank line. */
}
.rich-text p br {
  display: none;
}
.rich-text ul,
.rich-text ol {
  line-height: 0;
  margin: 0;
  padding: 0;
}
.rich-text ul p,
.rich-text ol p {
  /*
       * <p> tags in a list do not preserve whitespace, since there's
       * mandatory leading whitespace before the content.
       */
}
.rich-text ul p br,
.rich-text ol p br {
  display: block;
}
.rich-text ul ul,
.rich-text ol ul,
.rich-text ul ol,
.rich-text ol ol {
  margin-left: 2ch;
}
.rich-text ol {
  /*
     * We want to keep wrapped text within the confines of the list item. So:
     *
     * 1. This is long text that
     *    wraps like this.
     *
     * Instead of:
     *
     * 1. This is long text that
     * wraps like this.
     *
     * To do this, we need to use display: table and display: table-cell.
     * However, that loses item counter rendering, so we have to do counters
     * ourselves. In combination with StyledOListBlockProcessor in
     * reviewboard/reviews/markdown_extensions.py, this maintains compatibility
     * with the start= attribute.
     */
  counter-reset: li;
  display: table;
  list-style-position: inside;
}
.rich-text ol > li {
  counter-increment: li;
  display: table;
}
.rich-text ol > li::before {
  content: counter(li) ". ";
  padding-right: 1ch;
  display: table-cell;
  text-align: right;
}
.rich-text ul {
  list-style-position: outside;
  list-style: disc;
  margin-left: 2ch;
}
.rich-text li {
  line-height: 15px;
}
.rich-text li ~ li:last-child p:last-child {
  margin-bottom: 0;
}
.rich-text li p {
  margin-bottom: 15px;
  /*
       * If the list item ends up with a sole <p>, just remove the margins.
       * Depending on neighboring elements, Python Markdown will sometimes
       * create list items this way.
       */
}
.rich-text li p:first-child:last-child {
  margin: 0;
}
.rich-text table th,
.rich-text table td {
  border-bottom: 1px solid #bbb;
  padding: 0.2em 1em;
}
.rich-text code {
  color: #4444cc;
}
.rich-text pre {
  white-space: pre-wrap;
  /* CSS 3 */
  white-space: -moz-pre-wrap;
  /* Mozilla, 1999+ */
  white-space: -pre-wrap;
  /* Opera 4-6 */
  white-space: -o-pre-wrap;
  /* Opera 7 */
  word-wrap: break-word;
  /* IE 5.5+ */
  font-size: inherit;
  margin-left: 2.5em;
}
.rich-text pre * {
  white-space: pre-wrap;
  /* CSS 3 */
  white-space: -moz-pre-wrap;
  /* Mozilla, 1999+ */
  white-space: -pre-wrap;
  /* Opera 4-6 */
  white-space: -o-pre-wrap;
  /* Opera 7 */
  word-wrap: break-word;
  /* IE 5.5+ */
}
.rich-text blockquote {
  border-left: 1px solid #bbb;
  padding-left: 1em;
  margin-left: 0.5em;
}
.rich-text hr {
  border: 1px solid #ddd;
}
